package eu.eudat.commons.types.descriptiontemplate.fielddata;

import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.xml.XmlSerializable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class RadioBoxDataEntity extends BaseFieldDataEntity<RadioBoxDataEntity> {
    @Override
    public FieldType getFieldType() {
        return FieldType.RADIO_BOX;
    }
    public static class Option implements XmlSerializable<Option> {
        private String label;
        private String value;

        public String getLabel() {
            return label;
        }

        public void setLabel(String label) {
            this.label = label;
        }

        public String getValue() {
            return value;
        }

        public void setValue(String value) {
            this.value = value;
        }

        @Override
        public Element toXml(Document doc) {
            Element option = doc.createElement("option");
            option.setAttribute("label", this.label);
            option.setAttribute("value", this.value);
            return option;
        }

        @Override
        public Option fromXml(Element item) {
            this.label = item.getAttribute("label");
            this.value = item.getAttribute("value");
            return this;
        }


    }

    private List<Option> options;

    public List<Option> getOptions() {
        return options;
    }

    public void setOptions(List<Option> options) {
        this.options = options;
    }
    
    @Override
    public Element toXml(Document doc) {
        Element root = super.toXml(doc);
        Element element = doc.createElement("options");
        for (Option option : this.options) {
            element.appendChild(option.toXml(doc));
        }
        root.appendChild(element);
        return root;
    }

    @Override
    public RadioBoxDataEntity fromXml(Element item) {
        super.fromXml(item);
        this.options = new LinkedList<>();
        Element optionsElement = (Element) item.getElementsByTagName("options").item(0);
        if (optionsElement != null) {
            NodeList optionElements = optionsElement.getChildNodes();
            for (int temp = 0; temp < optionElements.getLength(); temp++) {
                Node optionElement = optionElements.item(temp);
                if (optionElement.getNodeType() == Node.ELEMENT_NODE) {
                    this.options.add(new Option().fromXml((Element) optionElement));
                }
            }
        }
        return this;
    }
}
