package eu.eudat.commons.types.dmpblueprint;

import eu.eudat.commons.types.xml.XmlBuilder;
import eu.eudat.commons.types.xml.XmlSerializable;
import jakarta.xml.bind.annotation.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.LinkedList;
import java.util.List;

@XmlRootElement(name = "root")
@XmlAccessorType(XmlAccessType.FIELD)
public class DefinitionEntity implements XmlSerializable<DefinitionEntity> {
	@XmlElementWrapper(name = "sections")
	@XmlElement(name = "section")
	private List<SectionEntity> sections;

	public List<SectionEntity> getSections() {
		return sections;
	}
	public void setSections(List<SectionEntity> sections) {
		this.sections = sections;
	}

	@Override
	public Element toXml(Document doc) {
		Element root = doc.createElement("root");
		Element sections = doc.createElement("sections");
		for (SectionEntity section : this.sections) {
			sections.appendChild(section.toXml(doc));
		}
		root.appendChild(sections);
		return root;
	}

	@Override
	public DefinitionEntity fromXml(Element item) {
		this.sections = new LinkedList<>();
		Element sections = (Element) XmlBuilder.getNodeFromListByTagName(item.getChildNodes(), "sections");
		if (sections != null) {
			NodeList sectionElements = sections.getChildNodes();
			for (int temp = 0; temp < sectionElements.getLength(); temp++) {
				Node sectionElement = sectionElements.item(temp);
				if (sectionElement.getNodeType() == Node.ELEMENT_NODE) {
					this.sections.add(new SectionEntity().fromXml((Element) sectionElement));
				}
			}
		}
		return this;
	}
	
}
