package eu.eudat.commons.types.dmpblueprint;

import eu.eudat.commons.enums.DmpBlueprintFieldCategory;
import eu.eudat.commons.types.xml.XmlBuilder;
import eu.eudat.commons.types.xml.XmlSerializable;
import jakarta.xml.bind.annotation.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.LinkedList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XmlAccessorType(XmlAccessType.FIELD)
public class SectionEntity implements XmlSerializable<SectionEntity> {

    @XmlAttribute(name="id")
    private UUID id;

    @XmlAttribute(name="label")
    private String label;

    @XmlAttribute(name="description")
    private String description;

    @XmlAttribute(name="ordinal")
    private Integer ordinal;

    @XmlTransient
    private List<FieldEntity> fields;

    @XmlAttribute(name="hasTemplates")
    private Boolean hasTemplates;
    
    @XmlElementWrapper(name = "descriptionTemplates")
    @XmlElement(name = "descriptionTemplate")
    private List<DescriptionTemplateEntity> descriptionTemplates;

    public UUID getId() {
        return id;
    }
    public void setId(UUID id) {
        this.id = id;
    }

    public String getLabel() {
        return label;
    }
    public void setLabel(String label) {
        this.label = label;
    }

    public String getDescription() {
        return description;
    }
    public void setDescription(String description) {
        this.description = description;
    }

    public Integer getOrdinal() {
        return ordinal;
    }
    public void setOrdinal(Integer ordinal) {
        this.ordinal = ordinal;
    }

    public List<FieldEntity> getFields() {
        return fields;
    }
    public void setFields(List<FieldEntity> fields) {
        this.fields = fields;
    }

    public Boolean getHasTemplates() {
        return hasTemplates;
    }
    public void setHasTemplates(Boolean hasTemplates) {
        this.hasTemplates = hasTemplates;
    }

    public List<DescriptionTemplateEntity> getDescriptionTemplates() {
        return descriptionTemplates;
    }
    public void setDescriptionTemplates(List<DescriptionTemplateEntity> descriptionTemplates) {
        this.descriptionTemplates = descriptionTemplates;
    }

    @Override
    public Element toXml(Document doc) {
        Element rootElement = doc.createElement("section");
        rootElement.setAttribute("id", this.getId().toString());
        rootElement.setAttribute("label", this.label);
        rootElement.setAttribute("description", this.description);
        rootElement.setAttribute("ordinal", String.valueOf(this.ordinal));
        rootElement.setAttribute("hasTemplates", String.valueOf(this.hasTemplates));
        List<FieldEntity> temp = this.fields.stream().filter(f -> f.getCategory().equals(DmpBlueprintFieldCategory.System)).collect(Collectors.toList());
        List<SystemFieldEntity> systemFieldsList = temp.stream().map(x-> (SystemFieldEntity)x).collect(Collectors.toList());
        Element systemFields = doc.createElement("systemFields");
        for (SystemFieldEntity systemField : systemFieldsList) {
            systemFields.appendChild(systemField.toXml(doc));
        }
        rootElement.appendChild(systemFields);
        if (this.descriptionTemplates != null) {
            Element descriptionTemplates = doc.createElement("descriptionTemplates");
            for (DescriptionTemplateEntity descriptionTemplate : this.descriptionTemplates) {
                descriptionTemplates.appendChild(descriptionTemplate.toXml(doc));
            }
            rootElement.appendChild(descriptionTemplates);
        }
        temp = this.fields.stream().filter(f -> f.getCategory().equals(DmpBlueprintFieldCategory.Extra)).collect(Collectors.toList());
        List<ExtraFieldEntity> extraFieldList = temp.stream().map(x-> (ExtraFieldEntity)x).collect(Collectors.toList());
        Element extraFields = doc.createElement("extraFields");
        for (ExtraFieldEntity extraField : extraFieldList) {
            extraFields.appendChild(extraField.toXml(doc));
        }
        rootElement.appendChild(extraFields);

        return rootElement;
    }

    @Override
    public SectionEntity fromXml(Element item) {
        this.id = UUID.fromString(item.getAttribute("id"));
        this.label = item.getAttribute("label");
        this.description = item.getAttribute("description");
        this.ordinal = Integer.valueOf(item.getAttribute("ordinal"));
        this.hasTemplates = Boolean.valueOf(item.getAttribute("hasTemplates"));
        this.fields = new LinkedList<>();
        Element systemFields = (Element) XmlBuilder.getNodeFromListByTagName(item.getChildNodes(), "systemFields");
        if (systemFields != null) {
            NodeList systemFieldElements = systemFields.getChildNodes();
            for (int temp = 0; temp < systemFieldElements.getLength(); temp++) {
                Node systemFieldElement = systemFieldElements.item(temp);
                if (systemFieldElement.getNodeType() == Node.ELEMENT_NODE) {
                    this.fields.add(new SystemFieldEntity().fromXml((Element) systemFieldElement));
                }
            }
        }
        this.descriptionTemplates = new LinkedList<>();
        Element descriptionTemplates = (Element) XmlBuilder.getNodeFromListByTagName(item.getChildNodes(), "descriptionTemplates");
        if (descriptionTemplates != null) {
            NodeList descriptionTemplateElements = descriptionTemplates.getChildNodes();
            for (int temp = 0; temp < descriptionTemplateElements.getLength(); temp++) {
                Node descriptionTemplateElement = descriptionTemplateElements.item(temp);
                if (descriptionTemplateElement.getNodeType() == Node.ELEMENT_NODE) {
                    this.descriptionTemplates.add(new DescriptionTemplateEntity().fromXml((Element) descriptionTemplateElement));
                }
            }
        }
        Element extraFields = (Element) XmlBuilder.getNodeFromListByTagName(item.getChildNodes(), "extraFields");
        if (extraFields != null) {
            NodeList extraFieldElements = extraFields.getChildNodes();
            for (int temp = 0; temp < extraFieldElements.getLength(); temp++) {
                Node extraFieldElement = extraFieldElements.item(temp);
                if (extraFieldElement.getNodeType() == Node.ELEMENT_NODE) {
                    this.fields.add(new ExtraFieldEntity().fromXml((Element) extraFieldElement));
                }
            }
        }
        return this;
    }
}
