package eu.eudat.model.builder.referencesearch;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.configurations.referencetype.ReferenceTypeProperties;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.BaseBuilder;
import eu.eudat.model.referencedefinition.Definition;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.logging.DataLogEntry;
import gr.cite.tools.logging.LoggerService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DefinitionSearchBuilder extends BaseBuilder<Definition, Map<String, String>> {

    private final BuilderFactory builderFactory;
    private final ReferenceTypeProperties properties;
    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    @Autowired
    public DefinitionSearchBuilder(
            ConventionService conventionService, BuilderFactory builderFactory, ReferenceTypeProperties properties) {
        super(conventionService, new LoggerService(LoggerFactory.getLogger(DefinitionSearchBuilder.class)));
        this.builderFactory = builderFactory;
        this.properties = properties;
    }

    public DefinitionSearchBuilder authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    @Override
    public List<Definition> build(FieldSet fields, List<Map<String, String>> data) throws MyApplicationException {
        this.logger.debug("building for {} items requesting {} fields", Optional.ofNullable(data).map(List::size).orElse(0), Optional.ofNullable(fields).map(FieldSet::getFields).map(Set::size).orElse(0));
        this.logger.trace(new DataLogEntry("requested fields", fields));
        if (data == null)
            return new ArrayList<>();
        
        List<Definition> models = new ArrayList<>();
        //for (Map<String, String> d : data) {
        Definition m = new Definition();
        m.setFields(this.builderFactory.builder(FieldSearchBuilder.class).authorize(this.authorize).build(null, data));
        models.add(m);
        //}
        this.logger.debug("build {} items", Optional.of(models).map(List::size).orElse(0));
        return models;
    }
}
