package eu.eudat.model.persist;

import eu.eudat.commons.enums.ActionConfirmationStatus;
import eu.eudat.commons.enums.ActionConfirmationType;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.actionconfirmation.DmpInvitationPersist;
import eu.eudat.model.persist.actionconfirmation.EmailConfirmationPersist;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

public class ActionConfirmationPersist {

    private UUID id;

    private ActionConfirmationType type;

    private static final String _type = "type";

    private ActionConfirmationStatus status;

    private static final String _status = "status";

    private String token;

    private static final String _token = "token";

    private DmpInvitationPersist dmpInvitation;

    private static final String _dmpInvitation = "dmpInvitation";

    private EmailConfirmationPersist emailConfirmation;

    private static final String _emailConfirmation = "emailConfirmation";

    private Instant expiresAt;

    private static final String _expiresAt = "expiresAt";

    private UUID createdById;

    private static final String _createdById = "createdById";

    private String hash;

    private static final String _hash = "hash";

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public ActionConfirmationType getType() {
        return type;
    }

    public void setType(ActionConfirmationType type) {
        this.type = type;
    }

    public ActionConfirmationStatus getStatus() {
        return status;
    }

    public void setStatus(ActionConfirmationStatus status) {
        this.status = status;
    }

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public DmpInvitationPersist getDmpInvitation() {
        return dmpInvitation;
    }

    public void setDmpInvitation(DmpInvitationPersist dmpInvitation) {
        this.dmpInvitation = dmpInvitation;
    }

    public EmailConfirmationPersist getEmailConfirmation() {
        return emailConfirmation;
    }

    public void setEmailConfirmation(EmailConfirmationPersist emailConfirmation) {
        this.emailConfirmation = emailConfirmation;
    }

    public Instant getExpiresAt() {
        return expiresAt;
    }

    public void setExpiresAt(Instant expiresAt) {
        this.expiresAt = expiresAt;
    }

    public UUID getCreatedById() {
        return createdById;
    }

    public void setCreatedById(UUID createdById) {
        this.createdById = createdById;
    }

    public String getHash() {
        return hash;
    }

    public void setHash(String hash) {
        this.hash = hash;
    }

    @Component(ActionConfirmationPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class ActionConfirmationPersistValidator extends BaseValidator<ActionConfirmationPersist> {

        public static final String ValidatorName = "ActionConfirmationPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        public ActionConfirmationPersistValidator(MessageSource messageSource, ConventionService conventionService, ErrorThesaurusProperties errors, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<ActionConfirmationPersist> modelClass() {
            return ActionConfirmationPersist.class;
        }

        @Override
        protected List<Specification> specifications(ActionConfirmationPersist item) {
            return Arrays.asList(
                    this.spec()
                            .iff(() -> this.isValidGuid(item.getId()))
                            .must(() -> this.isValidHash(item.getHash()))
                            .failOn(ActionConfirmationPersist._hash).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._hash}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .iff(() -> !this.isValidGuid(item.getId()))
                            .must(() -> !this.isValidHash(item.getHash()))
                            .failOn(ActionConfirmationPersist._hash).failWith(messageSource.getMessage("Validation_OverPosting", new Object[]{}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getType()))
                            .failOn(ActionConfirmationPersist._type).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._type}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getStatus()))
                            .failOn(ActionConfirmationPersist._status).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._status}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isEmpty(item.getToken()))
                            .failOn(ActionConfirmationPersist._token).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._token}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getExpiresAt()))
                            .failOn(ActionConfirmationPersist._expiresAt).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._expiresAt}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getCreatedById()))
                            .failOn(ActionConfirmationPersist._createdById).failWith(messageSource.getMessage("Validation_Required", new Object[]{ActionConfirmationPersist._createdById}, LocaleContextHolder.getLocale())),

                    this.refSpec()
                            .iff(() -> !this.isNull(item.getDmpInvitation()))
                            .on(ActionConfirmationPersist._dmpInvitation)
                            .over(item.getDmpInvitation())
                            .using(() -> this.validatorFactory.validator(DmpInvitationPersist.DmpInvitationPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getEmailConfirmation()))
                            .on(ActionConfirmationPersist._emailConfirmation)
                            .over(item.getEmailConfirmation())
                            .using(() -> this.validatorFactory.validator(EmailConfirmationPersist.EmailConfirmationPersistValidator.class))
            );
        }

    }

}



