package eu.eudat.model.persist.actionconfirmation;

import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

public class EmailConfirmationPersist {

    private String email;

    public static final String _email = "email";

    public EmailConfirmationPersist(String email) {
        this.email = email;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    @Component(EmailConfirmationPersistValidator.ValidatorName)
    @Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class EmailConfirmationPersistValidator extends BaseValidator<EmailConfirmationPersist> {

        public static final String ValidatorName = "EmailConfirmationPersistValidator";

        private final MessageSource messageSource;

        protected EmailConfirmationPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource) {
            super(conventionService, errors);
            this.messageSource = messageSource;
        }

        @Override
        protected Class<EmailConfirmationPersist> modelClass() {
            return EmailConfirmationPersist.class;
        }

        @Override
        protected List<Specification> specifications(EmailConfirmationPersist item) {
            return Collections.singletonList(
                    this.spec()
                            .must(() -> !this.isEmpty(item.getEmail()))
                            .failOn(EmailConfirmationPersist._email).failWith(messageSource.getMessage("Validation_Required", new Object[]{EmailConfirmationPersist._email}, LocaleContextHolder.getLocale()))
            );
        }
    }

}
