package eu.eudat.model.persist.descriptionproperties;

import eu.eudat.commons.enums.DescriptionStatus;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.validation.StatusAware;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class PropertyDefinitionPersist {

    private List<FieldPersist> fields;

    public static final String _fields = "fields";

    public List<FieldPersist> getFields() {
        return fields;
    }

    public void setFields(List<FieldPersist> fields) {
        this.fields = fields;
    }

    @Component(PropertyDefinitionPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class PropertyDefinitionPersistValidator extends BaseValidator<PropertyDefinitionPersist> implements StatusAware<DescriptionStatus> {

        public static final String ValidatorName = "Description.PropertyDefinitionPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        private DescriptionStatus status;

        protected PropertyDefinitionPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<PropertyDefinitionPersist> modelClass() {
            return PropertyDefinitionPersist.class;
        }

        @Override
        protected List<Specification> specifications(PropertyDefinitionPersist item) {
            return Arrays.asList(
                    this.spec()
                            .iff(() -> this.status == DescriptionStatus.Finalized)
                            .must(() -> !this.isNull(item.getFields()))
                            .failOn(PropertyDefinitionPersist._fields).failWith(messageSource.getMessage("Validation_Required", new Object[]{PropertyDefinitionPersist._fields}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getFields()))
                            .on(PropertyDefinitionPersist._fields)
                            .over(item.getFields())
                            .using(() -> this.validatorFactory.validator(FieldPersist.FieldPersistValidator.class))
            );
        }

        @Override
        public PropertyDefinitionPersistValidator setStatus(DescriptionStatus status) {
            this.status = status;
            return this;
        }
    }

}
