package eu.eudat.model.persist.descriptiontemplatedefinition;

import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class DefinitionPersist {

    private List<SectionPersist> sections = null;

    public static final String _sections = "sections";

    private List<PagePersist> pages = null;

    public static final String _pages = "pages";

    public List<SectionPersist> getSections() {
        return sections;
    }

    public void setSections(List<SectionPersist> sections) {
        this.sections = sections;
    }

    public List<PagePersist> getPages() {
        return pages;
    }

    public void setPages(List<PagePersist> pages) {
        this.pages = pages;
    }

    @Component(DefinitionPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class DefinitionPersistValidator extends BaseValidator<DefinitionPersist> {

        public static final String ValidatorName = "DescriptionTemplate.DefinitionPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        public DefinitionPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<DefinitionPersist> modelClass() {
            return DefinitionPersist.class;
        }

        @Override
        protected List<Specification> specifications(DefinitionPersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getSections()))
                            .failOn(DefinitionPersist._sections).failWith(messageSource.getMessage("Validation_Required", new Object[]{DefinitionPersist._sections}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getSections()))
                            .on(DefinitionPersist._sections)
                            .over(item.getSections())
                            .using(() -> this.validatorFactory.validator(SectionPersist.SectionPersistValidator.class)),
                    this.spec()
                            .must(() -> !this.isNull(item.getPages()))
                            .failOn(DefinitionPersist._pages).failWith(messageSource.getMessage("Validation_Required", new Object[]{DefinitionPersist._pages}, LocaleContextHolder.getLocale())),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getPages()))
                            .on(DefinitionPersist._pages)
                            .over(item.getPages())
                            .using(() -> this.validatorFactory.validator(PagePersist.PagePersistValidator.class))
            );
        }
    }

}
