package eu.eudat.model.persist.descriptiontemplatedefinition;

import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.enums.FieldValidationType;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class FieldPersist {

    private String id = null;

    public static final String _id = "id";

    private Integer ordinal = null;

    public static final String _ordinal = "ordinal";

    private List<String> schematics;

    private String defaultValue;

    private List<RulePersist> visibilityRules;

    public static final String _visibilityRules = "visibilityRules";

    private List<FieldValidationType> validations;

    private Boolean includeInExport = null;

    public static final String _includeInExport = "includeInExport";

    private BaseFieldDataPersist data = null;

    public static final String _data = "data";

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public Integer getOrdinal() {
        return ordinal;
    }

    public void setOrdinal(Integer ordinal) {
        this.ordinal = ordinal;
    }

    public List<String> getSchematics() {
        return schematics;
    }

    public void setSchematics(List<String> schematics) {
        this.schematics = schematics;
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public List<RulePersist> getVisibilityRules() {
        return visibilityRules;
    }

    public void setVisibilityRules(List<RulePersist> visibilityRules) {
        this.visibilityRules = visibilityRules;
    }

    public List<FieldValidationType> getValidations() {
        return validations;
    }

    public void setValidations(List<FieldValidationType> validations) {
        this.validations = validations;
    }

    public Boolean getIncludeInExport() {
        return includeInExport;
    }

    public void setIncludeInExport(Boolean includeInExport) {
        this.includeInExport = includeInExport;
    }

    public BaseFieldDataPersist getData() {
        return data;
    }

    public void setData(BaseFieldDataPersist data) {
        this.data = data;
    }

    @Component(FieldPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class FieldPersistValidator extends BaseValidator<FieldPersist> {

        public static final String ValidatorName = "DescriptionTemplate.FieldPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected FieldPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<FieldPersist> modelClass() {
            return FieldPersist.class;
        }

        @Override
        protected List<Specification> specifications(FieldPersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isEmpty(item.getId()))
                            .failOn(FieldPersist._id).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldPersist._id}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getOrdinal()))
                            .failOn(FieldPersist._ordinal).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldPersist._ordinal}, LocaleContextHolder.getLocale())),
                    this.spec()
                            .must(() -> !this.isNull(item.getIncludeInExport()))
                            .failOn(FieldPersist._includeInExport).failWith(messageSource.getMessage("Validation_Required", new Object[]{FieldPersist._includeInExport}, LocaleContextHolder.getLocale())),

                    this.navSpec()
                            .iff(() -> !this.isNull(item.getVisibilityRules()))
                            .on(FieldPersist._visibilityRules)
                            .over(item.getVisibilityRules())
                            .using(() -> this.validatorFactory.validator(RulePersist.RulePersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.AUTO_COMPLETE)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(AutoCompleteDataPersist.AutoCompleteDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.BOOLEAN_DECISION)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(BooleanDecisionDataPersist.BooleanDecisionPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.INTERNAL_DMP_ENTRIES_DATASETS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DatasetAutoCompleteDataPersist.DatasetAutoCompleteDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.INTERNAL_DMP_ENTRIES_DMPS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DmpAutoCompleteDataPersist.DmpAutoCompleteDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.CHECK_BOX)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(CheckBoxDataPersist.CheckBoxDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.DATE_PICKER)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DatePickerDataPersist.DatePickerDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.EXTERNAL_DATASETS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(ExternalDatasetDataPersist.ExternalDatasetDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.FREE_TEXT)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(FreeTextDataPersist.FreeTextDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.LICENSES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(LicenseDataPersist.LicenseDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.ORGANIZATIONS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(OrganizationDataPersist.OrganizationDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.PUBLICATIONS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(PublicationDataPersist.PublicationDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.RADIO_BOX)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(RadioBoxDataPersist.RadioBoxDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.REGISTRIES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(RegistryDataPersist.RegistryDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.INTERNAL_DMP_ENTRIES_RESEARCHERS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(ResearcherAutoCompleteDataPersist.ResearcherAutoCompleteDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.RESEARCHERS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(ResearcherDataPersist.ResearcherDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.RICH_TEXT_AREA)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(RichTextAreaDataPersist.RichTextAreaDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.SERVICES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(ServiceDataPersist.ServiceDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.TAGS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(TagDataPersist.TagDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.TAXONOMIES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(TaxonomyDataPersist.TaxonomyDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.TEXT_AREA)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(TextAreaDataPersist.TextAreaDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.UPLOAD)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(UploadDataPersist.UploadDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.VALIDATION)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(ValidationDataPersist.ValidationDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.DATASET_IDENTIFIER)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DatasetIdentifierDataPersist.DatasetIdentifierDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.CURRENCY)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(CurrencyDataPersist.CurrencyDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.WORD_LIST)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(WordListDataPersist.WordListDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.DATA_REPOSITORIES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(DataRepositoryDataPersist.DataRepositoryDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.JOURNAL_REPOSITORIES)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(JournalRepositoryDataPersist.JournalRepositoryDataPersistValidator.class)),
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getData()) && item.getData().getFieldType() == FieldType.PUBLICATIONS)
                            .on(FieldPersist._data)
                            .over(item.getData())
                            .using(() -> this.validatorFactory.validator(PublicationRepositoryDataPersist.PublicationRepositoryDataPersistValidator.class))
            );
        }
    }

}
