package eu.eudat.model.persist.descriptiontemplatedefinition.fielddata;

import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class RadioBoxDataPersist extends BaseFieldDataPersist {

    private List<RadioBoxOptionPersist> options = null;

    public static final String _options = "options";

    public List<RadioBoxOptionPersist> getOptions() {
        return options;
    }

    public void setOptions(List<RadioBoxOptionPersist> options) {
        this.options = options;
    }

    @Component(RadioBoxDataPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class RadioBoxDataPersistValidator extends BaseFieldDataPersistValidator<RadioBoxDataPersist> {

        public static final String ValidatorName = "DescriptionTemplate.RadioBoxDataPersistValidator";

        private final ValidatorFactory validatorFactory;

        protected RadioBoxDataPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors, messageSource);
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<RadioBoxDataPersist> modelClass() {
            return RadioBoxDataPersist.class;
        }

        @Override
        protected List<Specification> specifications(RadioBoxDataPersist item) {
            List<Specification> specifications = getBaseSpecifications(item);
            specifications.addAll(Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getOptions()))
                            .failOn(RadioBoxDataPersist._options).failWith(messageSource.getMessage("Validation_Required", new Object[]{RadioBoxDataPersist._options}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getOptions()))
                            .on(RadioBoxDataPersist._options)
                            .over(item.getOptions())
                            .using(() -> this.validatorFactory.validator(RadioBoxOptionPersist.RadioBoxOptionPersistValidator.class))
            ));
            return specifications;
        }
    }

}
