package eu.eudat.model.persist.descriptiontemplatedefinition.fielddata;

import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class UploadDataPersist extends BaseFieldDataPersist {

    private List<UploadOptionPersist> types = null;

    public static final String _types = "types";

    public List<UploadOptionPersist> getTypes() {
        return types;
    }

    public void setTypes(List<UploadOptionPersist> types) {
        this.types = types;
    }

    private Integer maxFileSizeInMB;

    public static final String _maxFileSizeInMB = "maxFileSizeInMB";

    public Integer getMaxFileSizeInMB() {
        return maxFileSizeInMB;
    }

    public void setMaxFileSizeInMB(Integer maxFileSizeInMB) {
        this.maxFileSizeInMB = maxFileSizeInMB;
    }

    @Component(UploadDataPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class UploadDataPersistValidator extends BaseFieldDataPersistValidator<UploadDataPersist> {

        public static final String ValidatorName = "DescriptionTemplate.UploadDataPersistValidator";

        private final ValidatorFactory validatorFactory;

        protected UploadDataPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors, messageSource);
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<UploadDataPersist> modelClass() {
            return UploadDataPersist.class;
        }

        @Override
        protected List<Specification> specifications(UploadDataPersist item) {
            List<Specification> specifications = getBaseSpecifications(item);
            specifications.addAll(Arrays.asList(
                    this.spec()
                            .iff(() -> !this.isNull(item.getMaxFileSizeInMB()))
                            .must(() -> !this.lessEqual(item.getMaxFileSizeInMB(), Integer.MAX_VALUE))
                            .failOn(UploadDataPersist._maxFileSizeInMB).failWith(messageSource.getMessage("Validation_MaxLength", new Object[]{UploadDataPersist._types}, LocaleContextHolder.getLocale())),

                    this.spec()
                            .must(() -> !this.isNull(item.getTypes()))
                            .failOn(UploadDataPersist._types).failWith(messageSource.getMessage("Validation_Required", new Object[]{UploadDataPersist._types}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getTypes()))
                            .on(UploadDataPersist._types)
                            .over(item.getTypes())
                            .using(() -> this.validatorFactory.validator(UploadOptionPersist.UploadOptionPersistValidator.class))
            ));
            return specifications;
        }
    }

}

