package eu.eudat.model.persist.dmpblueprintdefinition;

import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class DefinitionPersist {

	private List<SectionPersist> sections = null;

	public static final String _sections = "sections";

	public List<SectionPersist> getSections() {
		return sections;
	}

	public void setSections(List<SectionPersist> sections) {
		this.sections = sections;
	}

	@Component(DefinitionPersistValidator.ValidatorName)
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public static class DefinitionPersistValidator extends BaseValidator<DefinitionPersist> {

		public static final String ValidatorName = "DmpBlueprint.DefinitionPersistValidator";

		private final MessageSource messageSource;

		private final ValidatorFactory validatorFactory;

		protected DefinitionPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
			super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

		@Override
		protected Class<DefinitionPersist> modelClass() {
			return DefinitionPersist.class;
		}

		@Override
		protected List<Specification> specifications(DefinitionPersist item) {
			return Arrays.asList(
					this.spec()
							.must(() -> !this.isNull(item.getSections()))
							.failOn(DefinitionPersist._sections).failWith(messageSource.getMessage("Validation_Required", new Object[]{eu.eudat.model.persist.descriptiontemplatedefinition.DefinitionPersist._sections}, LocaleContextHolder.getLocale())),
					this.refSpec()
							.iff(() -> !this.isNull(item.getSections()))
							.on(DefinitionPersist._sections)
							.over(item.getSections())
							.using(() -> this.validatorFactory.validator(SectionPersist.SectionPersistValidator.class))
			);
		}
	}

}
