package eu.eudat.model.persist.referencetypedefinition;

import eu.eudat.commons.enums.ReferenceTypeSourceType;
import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import eu.eudat.model.persist.DescriptionPersist;
import eu.eudat.model.persist.descriptionproperties.PropertyDefinitionPersist;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

public class ReferenceTypeDefinitionPersist {

    private List<ReferenceTypeFieldPersist> fields = null;

    public static final String _fields = "fields";

    private List<ReferenceTypeSourceBaseConfigurationPersist> sources = null;

    public static final String _sources = "sources";

    public List<ReferenceTypeFieldPersist> getFields() {
        return fields;
    }

    public void setFields(List<ReferenceTypeFieldPersist> fields) {
        this.fields = fields;
    }

    public List<ReferenceTypeSourceBaseConfigurationPersist> getSources() {
        return sources;
    }

    public void setSources(List<ReferenceTypeSourceBaseConfigurationPersist> sources) {
        this.sources = sources;
    }

    @Component(ReferenceTypeDefinitionPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class ReferenceTypeDefinitionPersistValidator extends BaseValidator<ReferenceTypeDefinitionPersist> {

        public static final String ValidatorName = "ReferenceTypeDefinitionPersistValidator";

        private final MessageSource messageSource;

        private final ValidatorFactory validatorFactory;

        protected ReferenceTypeDefinitionPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, MessageSource messageSource, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.messageSource = messageSource;
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<ReferenceTypeDefinitionPersist> modelClass() {
            return ReferenceTypeDefinitionPersist.class;
        }

        @Override
        protected List<Specification> specifications(ReferenceTypeDefinitionPersist item) {
            return Arrays.asList(
                    this.spec()
                            .must(() -> !this.isNull(item.getFields()))
                            .failOn(ReferenceTypeDefinitionPersist._fields).failWith(messageSource.getMessage("Validation_Required", new Object[]{ReferenceTypeDefinitionPersist._fields}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getFields()))
                            .on(ReferenceTypeDefinitionPersist._fields)
                            .over(item.getFields())
                            .using(() -> this.validatorFactory.validator(ReferenceTypeFieldPersist.ReferenceTypeFieldPersistValidator.class)),
                    this.spec()
                            .must(() -> !this.isNull(item.getSources()))
                            .failOn(ReferenceTypeDefinitionPersist._sources).failWith(messageSource.getMessage("Validation_Required", new Object[]{ReferenceTypeDefinitionPersist._sources}, LocaleContextHolder.getLocale())),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getSources()))
                            .on(ReferenceTypeDefinitionPersist._sources)
                            .over(item.getSources().stream().filter(x -> x.getType() == ReferenceTypeSourceType.API).toList())
                            .using(() -> this.validatorFactory.validator(ReferenceTypeSourceExternalApiConfigurationPersist.ReferenceTypeSourceExternalApiConfigurationPersistValidator.class)),
                    this.navSpec()
                            .iff(() -> !this.isNull(item.getSources()))
                            .on(ReferenceTypeDefinitionPersist._sources)
                            .over(item.getSources().stream().filter(x -> x.getType() == ReferenceTypeSourceType.STATIC).toList())
                            .using(() -> this.validatorFactory.validator(ReferenceTypeSourceStaticOptionConfigurationPersist.ReferenceTypeSourceStaticOptionConfigurationPersistValidator.class))
            );
        }
    }

}
