package eu.eudat.model.persist.tenantconfig;

import eu.eudat.commons.validation.BaseValidator;
import eu.eudat.commons.validation.ValidatorFactory;
import eu.eudat.commons.validation.specification.Specification;
import eu.eudat.convention.ConventionService;
import eu.eudat.errorcode.ErrorThesaurusProperties;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

public class TenantFileTransformersConfigPersist {

    private List<TenantSourcePersist> sources;

    public static final String _sources = "sources";

    public List<TenantSourcePersist> getSources() {
        return sources;
    }

    public void setSources(List<TenantSourcePersist> sources) {
        this.sources = sources;
    }

    @Component(TenantFileTransformersConfigPersistValidator.ValidatorName)
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public static class TenantFileTransformersConfigPersistValidator extends BaseValidator<TenantFileTransformersConfigPersist> {

        public static final String ValidatorName = "TenantFileTransformersConfigPersistValidator";

        private final ValidatorFactory validatorFactory;

        protected TenantFileTransformersConfigPersistValidator(ConventionService conventionService, ErrorThesaurusProperties errors, ValidatorFactory validatorFactory) {
            super(conventionService, errors);
            this.validatorFactory = validatorFactory;
        }

        @Override
        protected Class<TenantFileTransformersConfigPersist> modelClass() {
            return TenantFileTransformersConfigPersist.class;
        }

        @Override
        protected List<Specification> specifications(TenantFileTransformersConfigPersist item) {
            return Collections.singletonList(
                    this.refSpec()
                            .iff(() -> !this.isNull(item.getSources()))
                            .on(TenantFileTransformersConfigPersist._sources)
                            .over(item.getSources())
                            .using(() -> this.validatorFactory.validator(TenantSourcePersist.TenantSourcePersistValidator.class))
            );
        }
    }

}
