package eu.eudat.query;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.DmpBlueprintStatus;
import eu.eudat.commons.enums.IsActive;
import eu.eudat.commons.scope.user.UserScope;
import eu.eudat.data.DescriptionTemplateEntity;
import eu.eudat.data.DmpBlueprintEntity;
import eu.eudat.model.DescriptionTemplateType;
import eu.eudat.model.DmpBlueprint;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.data.query.FieldResolver;
import gr.cite.tools.data.query.QueryBase;
import gr.cite.tools.data.query.QueryContext;
import jakarta.persistence.Tuple;
import jakarta.persistence.criteria.CriteriaBuilder;
import jakarta.persistence.criteria.Predicate;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.*;

@Component
@Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE)
public class DmpBlueprintQuery extends QueryBase<DmpBlueprintEntity> {

    private String like;

    private Collection<UUID> ids;

    private Collection<IsActive> isActives;

    private Collection<DmpBlueprintStatus> statuses;

    private Collection<UUID> excludedIds;

    private EnumSet<AuthorizationFlags> authorize = EnumSet.of(AuthorizationFlags.None);

    public DmpBlueprintQuery like(String value) {
        this.like = value;
        return this;
    }

    public DmpBlueprintQuery ids(UUID value) {
        this.ids = List.of(value);
        return this;
    }

    public DmpBlueprintQuery ids(UUID... value) {
        this.ids = Arrays.asList(value);
        return this;
    }

    public DmpBlueprintQuery ids(Collection<UUID> values) {
        this.ids = values;
        return this;
    }

    public DmpBlueprintQuery isActive(IsActive value) {
        this.isActives = List.of(value);
        return this;
    }

    public DmpBlueprintQuery isActive(IsActive... value) {
        this.isActives = Arrays.asList(value);
        return this;
    }

    public DmpBlueprintQuery isActive(Collection<IsActive> values) {
        this.isActives = values;
        return this;
    }

    public DmpBlueprintQuery statuses(DmpBlueprintStatus value) {
        this.statuses = List.of(value);
        return this;
    }

    public DmpBlueprintQuery statuses(DmpBlueprintStatus... value) {
        this.statuses = Arrays.asList(value);
        return this;
    }

    public DmpBlueprintQuery statuses(Collection<DmpBlueprintStatus> values) {
        this.statuses = values;
        return this;
    }

    public DmpBlueprintQuery excludedIds(Collection<UUID> values) {
        this.excludedIds = values;
        return this;
    }

    public DmpBlueprintQuery excludedIds(UUID value) {
        this.excludedIds = List.of(value);
        return this;
    }

    public DmpBlueprintQuery excludedIds(UUID... value) {
        this.excludedIds = Arrays.asList(value);
        return this;
    }

    public DmpBlueprintQuery authorize(EnumSet<AuthorizationFlags> values) {
        this.authorize = values;
        return this;
    }

    public DmpBlueprintQuery(
            UserScope userScope,
            AuthorizationService authService
    ) {
    }

    @Override
    protected Class<DmpBlueprintEntity> entityClass() {
        return DmpBlueprintEntity.class;
    }

    @Override
    protected Boolean isFalseQuery() {
        return this.isEmpty(this.ids) || this.isEmpty(this.isActives) || this.isEmpty(this.excludedIds) || this.isEmpty(this.statuses);
    }

    @Override
    protected <X, Y> Predicate applyFilters(QueryContext<X, Y> queryContext) {
        List<Predicate> predicates = new ArrayList<>();
        if (this.ids != null) {
            CriteriaBuilder.In<UUID> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpBlueprintEntity._id));
            for (UUID item : this.ids)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.like != null && !this.like.isEmpty()) {
            predicates.add(queryContext.CriteriaBuilder.like(queryContext.Root.get(DmpBlueprintEntity._label), this.like));
        }
        if (this.isActives != null) {
            CriteriaBuilder.In<IsActive> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpBlueprintEntity._isActive));
            for (IsActive item : this.isActives)
                inClause.value(item);
            predicates.add(inClause);
        }

        if (this.statuses != null) {
            CriteriaBuilder.In<DmpBlueprintStatus> inClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpBlueprintEntity._status));
            for (DmpBlueprintStatus item : this.statuses)
                inClause.value(item);
            predicates.add(inClause);
        }
        if (this.excludedIds != null) {
            CriteriaBuilder.In<UUID> notInClause = queryContext.CriteriaBuilder.in(queryContext.Root.get(DmpBlueprintEntity._id));
            for (UUID item : this.excludedIds)
                notInClause.value(item);
            predicates.add(notInClause.not());
        }
        if (!predicates.isEmpty()) {
            Predicate[] predicatesArray = predicates.toArray(new Predicate[0]);
            return queryContext.CriteriaBuilder.and(predicatesArray);
        } else {
            return null;
        }
    }

    @Override
    protected DmpBlueprintEntity convert(Tuple tuple, Set<String> columns) {
        DmpBlueprintEntity item = new DmpBlueprintEntity();
        item.setId(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._id, UUID.class));
        item.setLabel(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._label, String.class));
        item.setCreatedAt(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._createdAt, Instant.class));
        item.setUpdatedAt(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._updatedAt, Instant.class));
        item.setIsActive(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._isActive, IsActive.class));
        item.setDefinition(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._definition, String.class));
        item.setStatus(QueryBase.convertSafe(tuple, columns, DmpBlueprintEntity._status, DmpBlueprintStatus.class));
        return item;
    }

    @Override
    protected String fieldNameOf(FieldResolver item) {
        if (item.match(DmpBlueprint._id)) return DmpBlueprintEntity._id;
        else if (item.match(DmpBlueprint._label)) return DmpBlueprintEntity._label;
        else if (item.match(DmpBlueprint._definition)) return DmpBlueprintEntity._definition;
        else if (item.prefix(DmpBlueprint._definition)) return DmpBlueprintEntity._definition;
        else if (item.match(DmpBlueprint._createdAt)) return DmpBlueprintEntity._createdAt;
        else if (item.match(DmpBlueprint._updatedAt)) return DmpBlueprintEntity._updatedAt;
        else if (item.match(DmpBlueprint._hash)) return DmpBlueprintEntity._updatedAt;
        else if (item.match(DmpBlueprint._isActive)) return DmpBlueprintEntity._isActive;
        else if (item.match(DmpBlueprint._status)) return DmpBlueprintEntity._status;
        else return null;
    }

}
