package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.SectionEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.AutoCompleteDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ComboBoxDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.*;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.AutoCompleteDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.AutoCompleteData;
import eu.eudat.model.persist.descriptiontemplatedefinition.FieldSetPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.PagePersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.SectionPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;
import org.w3c.dom.Element;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class AutoCompleteFieldDataHelperService extends BaseFieldDataHelperService<AutoCompleteData, AutoCompleteDataPersist, AutoCompleteDataEntity, AutoCompleteDataImportExport> {
	private final ConventionService conventionService;
	private final BuilderFactory builderFactory;

	public AutoCompleteFieldDataHelperService(ConventionService conventionService, BuilderFactory builderFactory) {
		this.conventionService = conventionService;
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.AUTO_COMPLETE;
	}

	@Override
	public AutoCompleteDataEntity newDataInstanceInternal() {
		return new AutoCompleteDataEntity();
	}

	@Override
	public AutoCompleteData newModelInstanceInternal() {
		return new AutoCompleteData();
	}

	@Override
	public AutoCompleteDataPersist newPersistModelInstanceInternal() {
		return new AutoCompleteDataPersist();
	}

	@Override
	protected AutoCompleteDataImportExport newImportExportInstanceInternal() {
		return new AutoCompleteDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return AutoCompleteDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return AutoCompleteData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return AutoCompleteDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return AutoCompleteDataImportExport.class;
	}

	@Override
	public List<AutoCompleteData> buildInternal(FieldSet fieldSet, List<AutoCompleteDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(AutoCompleteDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected AutoCompleteDataEntity applyPersistInternal(AutoCompleteDataPersist persist, AutoCompleteDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		if (!this.conventionService.isListNullOrEmpty(persist.getAutoCompleteSingleDataList())){
			data.setAutoCompleteSingleDataList(new ArrayList<>());
			for (AutoCompleteSingleDataPersist autoCompleteSingleDataPersist: persist.getAutoCompleteSingleDataList()) {
				data.getAutoCompleteSingleDataList().add(this.buildAutoCompleteSingleData(autoCompleteSingleDataPersist));
			}
		}
		return data;
	}

	private @NotNull AutoCompleteDataEntity.AutoCompleteSingleData buildAutoCompleteSingleData(AutoCompleteSingleDataPersist persist){
		AutoCompleteDataEntity.AutoCompleteSingleData data = new AutoCompleteDataEntity.AutoCompleteSingleData();
		if (persist == null) return data;

		data.setUrl(persist.getUrl());
		data.setAutocompleteType(persist.getAutocompleteType());
		data.setOptionsRoot(persist.getOptionsRoot());
		data.setHasAuth(persist.getHasAuth());
		if(persist.getAutoCompleteOptions() != null) data.setAutoCompleteOptions(this.buildOption(persist.getAutoCompleteOptions()));
		if(persist.getAuth() != null) data.setAuth(this.buildAuthAutoCompleteData(persist.getAuth()));
		data.setMethod(persist.getMethod());
		
		return data;
	}

	private @NotNull ComboBoxDataEntity.Option buildOption(ComboBoxOptionPersist persist){
		ComboBoxDataEntity.Option data = new ComboBoxDataEntity.Option();
		if (persist == null) return data;

		data.setLabel(persist.getLabel());
		data.setValue(persist.getValue());
		data.setUri(persist.getUri());
		data.setSource(persist.getSource());

		return data;
	}

	private @NotNull AutoCompleteDataEntity.AuthAutoCompleteData buildAuthAutoCompleteData(AuthAutoCompleteDataPersist persist){
		AutoCompleteDataEntity.AuthAutoCompleteData data = new AutoCompleteDataEntity.AuthAutoCompleteData();
		if (persist == null) return data;

		data.setBody(persist.getBody());
		data.setUrl(persist.getUrl());
		data.setMethod(persist.getMethod());
		data.setPath(persist.getPath());
		data.setType(persist.getType());

		return data;
	}

	@Override
	protected AutoCompleteDataPersist importExportMapDataToPersistInternal(AutoCompleteDataImportExport data, AutoCompleteDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		if (!this.conventionService.isListNullOrEmpty(data.getAutoCompleteSingleDataList())){
			persist.setAutoCompleteSingleDataList(new ArrayList<>());
			for (AutoCompleteSingleData autoCompleteSingleData: data.getAutoCompleteSingleDataList()) {
				persist.getAutoCompleteSingleDataList().add(this.buildAutoCompleteSingleData(autoCompleteSingleData));
			}
		}
		return persist;
	}

	private @NotNull AutoCompleteSingleDataPersist buildAutoCompleteSingleData(AutoCompleteSingleData data){
		AutoCompleteSingleDataPersist persist = new AutoCompleteSingleDataPersist();
		if (data == null) return persist;

		persist.setUrl(data.getUrl());
		persist.setAutocompleteType(data.getAutocompleteType());
		persist.setOptionsRoot(data.getOptionsRoot());
		persist.setHasAuth(data.getHasAuth());
		if(data.getAutoCompleteOptions() != null) persist.setAutoCompleteOptions(this.buildOption(data.getAutoCompleteOptions()));
		if(data.getAuth() != null) persist.setAuth(this.buildAuthAutoCompleteData(data.getAuth()));
		persist.setMethod(data.getMethod());

		return persist;
	}

	private @NotNull ComboBoxOptionPersist buildOption(ComboBoxDataOptionEntity data){
		ComboBoxOptionPersist persist = new ComboBoxOptionPersist();
		if (data == null) return persist;

		persist.setLabel(data.getLabel());
		persist.setValue(data.getValue());
		persist.setUri(data.getUri());
		persist.setSource(data.getSource());

		return persist;
	}

	private @NotNull AuthAutoCompleteDataPersist buildAuthAutoCompleteData(AuthAutoCompleteData data){
		AuthAutoCompleteDataPersist persist = new AuthAutoCompleteDataPersist();
		if (data == null) return persist;

		persist.setBody(data.getBody());
		persist.setUrl(data.getUrl());
		persist.setMethod(data.getMethod());
		persist.setPath(data.getPath());
		persist.setType(data.getType());

		return persist;
	}

	@Override
	protected AutoCompleteDataImportExport dataToImportExportXmlInternal(AutoCompleteDataEntity data, AutoCompleteDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		if (!this.conventionService.isListNullOrEmpty(data.getAutoCompleteSingleDataList())){
			xml.setAutoCompleteSingleDataList(new ArrayList<>());
			for (AutoCompleteDataEntity.AutoCompleteSingleData autoCompleteSingleData: data.getAutoCompleteSingleDataList()) {
				xml.getAutoCompleteSingleDataList().add(this.buildAutoCompleteSingleData(autoCompleteSingleData));
			}
		}
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(AutoCompleteDataEntity data) {
		return data.getMultiAutoComplete();
	}

	private @NotNull AutoCompleteSingleData buildAutoCompleteSingleData(AutoCompleteDataEntity.AutoCompleteSingleData data){
		AutoCompleteSingleData xml = new AutoCompleteSingleData();
		if (data == null) return xml;

		xml.setUrl(data.getUrl());
		xml.setAutocompleteType(data.getAutocompleteType());
		xml.setOptionsRoot(data.getOptionsRoot());
		xml.setHasAuth(data.getHasAuth());
		if(data.getAutoCompleteOptions() != null) xml.setAutoCompleteOptions(this.buildOption(data.getAutoCompleteOptions()));
		if(data.getAuth() != null) xml.setAuth(this.buildAuthAutoCompleteData(data.getAuth()));
		xml.setMethod(data.getMethod());

		return xml;
	}

	private @NotNull ComboBoxDataOptionEntity buildOption(ComboBoxDataEntity.Option data){
		ComboBoxDataOptionEntity xml = new ComboBoxDataOptionEntity();
		if (data == null) return xml;

		xml.setLabel(data.getLabel());
		xml.setValue(data.getValue());
		xml.setUri(data.getUri());
		xml.setSource(data.getSource());

		return xml;
	}

	private @NotNull AuthAutoCompleteData buildAuthAutoCompleteData(AutoCompleteDataEntity.AuthAutoCompleteData data){
		AuthAutoCompleteData xml = new AuthAutoCompleteData();
		if (data == null) return xml;

		xml.setBody(data.getBody());
		xml.setUrl(data.getUrl());
		xml.setMethod(data.getMethod());
		xml.setPath(data.getPath());
		xml.setType(data.getType());

		return xml;
	}
}
