package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BooleanDecisionDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.CheckBoxDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.CheckBoxDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.CheckBoxDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.CheckBoxData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BooleanDecisionDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.CheckBoxDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class CheckBoxFieldDataHelperService extends BaseFieldDataHelperService<CheckBoxData, CheckBoxDataPersist, CheckBoxDataEntity, CheckBoxDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public CheckBoxFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.CHECK_BOX;
	}

	@Override
	public CheckBoxDataEntity newDataInstanceInternal() {
		return new CheckBoxDataEntity();
	}

	@Override
	public CheckBoxData newModelInstanceInternal() {
		return new CheckBoxData();
	}

	@Override
	public CheckBoxDataPersist newPersistModelInstanceInternal() {
		return new CheckBoxDataPersist();
	}

	@Override
	protected CheckBoxDataImportExport newImportExportInstanceInternal() {
		return new CheckBoxDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return CheckBoxDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return CheckBoxData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return CheckBoxDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return CheckBoxDataImportExport.class;
	}

	@Override
	public List<CheckBoxData> buildInternal(FieldSet fieldSet, List<CheckBoxDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(CheckBoxDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected CheckBoxDataEntity applyPersistInternal(CheckBoxDataPersist persist, CheckBoxDataEntity data) {
		return data;
	}

	@Override
	protected CheckBoxDataPersist importExportMapDataToPersistInternal(CheckBoxDataImportExport data, CheckBoxDataPersist persist) {
		return persist;
	}

	@Override
	protected CheckBoxDataImportExport dataToImportExportXmlInternal(CheckBoxDataEntity data, CheckBoxDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(CheckBoxDataEntity data) {
		return false;
	}
}
