package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.CurrencyDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.CurrencyDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.CurrencyDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.CurrencyData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.CurrencyDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class CurrencyFieldDataHelperService extends BaseFieldDataHelperService<CurrencyData, CurrencyDataPersist, CurrencyDataEntity, CurrencyDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public CurrencyFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.CURRENCY;
	}

	@Override
	public CurrencyDataEntity newDataInstanceInternal() {
		return new CurrencyDataEntity();
	}

	@Override
	public CurrencyData newModelInstanceInternal() {
		return new CurrencyData();
	}

	@Override
	public CurrencyDataPersist newPersistModelInstanceInternal() {
		return new CurrencyDataPersist();
	}

	@Override
	protected CurrencyDataImportExport newImportExportInstanceInternal() {
		return new CurrencyDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return CurrencyDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return CurrencyData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return CurrencyDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return CurrencyDataImportExport.class;
	}

	@Override
	public List<CurrencyData> buildInternal(FieldSet fieldSet, List<CurrencyDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(CurrencyDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}
	@Override
	protected CurrencyDataEntity applyPersistInternal(CurrencyDataPersist persist, CurrencyDataEntity data) {
		return data;
	}

	@Override
	protected CurrencyDataPersist importExportMapDataToPersistInternal(CurrencyDataImportExport data, CurrencyDataPersist persist) {
		return persist;
	}

	@Override
	protected CurrencyDataImportExport dataToImportExportXmlInternal(CurrencyDataEntity data, CurrencyDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(CurrencyDataEntity data) {
		return false;
	}
}
