package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.AutoCompleteDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.DataRepositoryDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.DataRepositoryDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.DataRepositoryData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class DataRepositoryFieldDataHelperService extends BaseFieldDataHelperService<DataRepositoryData, DataRepositoryDataPersist, DataRepositoryDataEntity, DataRepositoryDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public DataRepositoryFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.DATA_REPOSITORIES;
	}

	@Override
	public DataRepositoryDataEntity newDataInstanceInternal() {
		return new DataRepositoryDataEntity();
	}

	@Override
	public DataRepositoryData newModelInstanceInternal() {
		return new DataRepositoryData();
	}

	@Override
	public DataRepositoryDataPersist newPersistModelInstanceInternal() {
		return new DataRepositoryDataPersist();
	}

	@Override
	protected DataRepositoryDataImportExport newImportExportInstanceInternal() {
		return new DataRepositoryDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return DataRepositoryDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return DataRepositoryData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return DataRepositoryDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return DataRepositoryDataImportExport.class;
	}

	@Override
	public List<DataRepositoryData> buildInternal(FieldSet fieldSet, List<DataRepositoryDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(DataRepositoryDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}	
	
	@Override
	protected DataRepositoryDataEntity applyPersistInternal(DataRepositoryDataPersist persist, DataRepositoryDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected DataRepositoryDataPersist importExportMapDataToPersistInternal(DataRepositoryDataImportExport data, DataRepositoryDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected DataRepositoryDataImportExport dataToImportExportXmlInternal(DataRepositoryDataEntity data, DataRepositoryDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(DataRepositoryDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
