package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DmpAutoCompleteDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.DmpAutoCompleteDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.DmpAutoCompleteDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.DmpAutoCompleteData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DmpAutoCompleteDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class DmpAutoCompleteFieldDataHelperService extends BaseFieldDataHelperService<DmpAutoCompleteData, DmpAutoCompleteDataPersist, DmpAutoCompleteDataEntity, DmpAutoCompleteDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public DmpAutoCompleteFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.INTERNAL_DMP_ENTRIES_DMPS;
	}

	@Override
	public DmpAutoCompleteDataEntity newDataInstanceInternal() {
		return new DmpAutoCompleteDataEntity();
	}

	@Override
	public DmpAutoCompleteData newModelInstanceInternal() {
		return new DmpAutoCompleteData();
	}

	@Override
	public DmpAutoCompleteDataPersist newPersistModelInstanceInternal() {
		return new DmpAutoCompleteDataPersist();
	}

	@Override
	protected DmpAutoCompleteDataImportExport newImportExportInstanceInternal() {
		return new DmpAutoCompleteDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return DmpAutoCompleteDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return DmpAutoCompleteData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return DmpAutoCompleteDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return DmpAutoCompleteDataImportExport.class;
	}

	@Override
	public List<DmpAutoCompleteData> buildInternal(FieldSet fieldSet, List<DmpAutoCompleteDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(DmpAutoCompleteDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected DmpAutoCompleteDataEntity applyPersistInternal(DmpAutoCompleteDataPersist persist, DmpAutoCompleteDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected DmpAutoCompleteDataPersist importExportMapDataToPersistInternal(DmpAutoCompleteDataImportExport data, DmpAutoCompleteDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected DmpAutoCompleteDataImportExport dataToImportExportXmlInternal(DmpAutoCompleteDataEntity data, DmpAutoCompleteDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(DmpAutoCompleteDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
