package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ExternalDatasetDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ExternalDatasetDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ExternalDatasetDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ExternalDatasetData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.ExternalDatasetDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class ExternalDatasetFieldDataHelperService extends BaseFieldDataHelperService<ExternalDatasetData, ExternalDatasetDataPersist, ExternalDatasetDataEntity, ExternalDatasetDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public ExternalDatasetFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.EXTERNAL_DATASETS;
	}

	@Override
	public ExternalDatasetDataEntity newDataInstanceInternal() {
		return new ExternalDatasetDataEntity();
	}

	@Override
	public ExternalDatasetData newModelInstanceInternal() {
		return new ExternalDatasetData();
	}

	@Override
	public ExternalDatasetDataPersist newPersistModelInstanceInternal() {
		return new ExternalDatasetDataPersist();
	}

	@Override
	protected ExternalDatasetDataImportExport newImportExportInstanceInternal() {
		return new ExternalDatasetDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ExternalDatasetDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ExternalDatasetData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ExternalDatasetDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ExternalDatasetDataImportExport.class;
	}

	@Override
	public List<ExternalDatasetData> buildInternal(FieldSet fieldSet, List<ExternalDatasetDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ExternalDatasetDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ExternalDatasetDataEntity applyPersistInternal(ExternalDatasetDataPersist persist, ExternalDatasetDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		data.setType(persist.getType());
		return data;
	}

	@Override
	protected ExternalDatasetDataPersist importExportMapDataToPersistInternal(ExternalDatasetDataImportExport data, ExternalDatasetDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		persist.setType(data.getType());
		return persist;
	}

	@Override
	protected ExternalDatasetDataImportExport dataToImportExportXmlInternal(ExternalDatasetDataEntity data, ExternalDatasetDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		xml.setType(data.getType());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ExternalDatasetDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
