package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BooleanDecisionDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.FreeTextDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.FreeTextDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.FreeTextDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.FreeTextData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BooleanDecisionDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.FreeTextDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class FreeTextFieldDataHelperService extends BaseFieldDataHelperService<FreeTextData, FreeTextDataPersist, FreeTextDataEntity, FreeTextDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public FreeTextFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.FREE_TEXT;
	}

	@Override
	public FreeTextDataEntity newDataInstanceInternal() {
		return new FreeTextDataEntity();
	}

	@Override
	public FreeTextData newModelInstanceInternal() {
		return new FreeTextData();
	}

	@Override
	public FreeTextDataPersist newPersistModelInstanceInternal() {
		return new FreeTextDataPersist();
	}

	@Override
	protected FreeTextDataImportExport newImportExportInstanceInternal() {
		return new FreeTextDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return FreeTextDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return FreeTextData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return FreeTextDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return FreeTextDataImportExport.class;
	}

	@Override
	public List<FreeTextData> buildInternal(FieldSet fieldSet, List<FreeTextDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(FreeTextDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected FreeTextDataEntity applyPersistInternal(FreeTextDataPersist persist, FreeTextDataEntity data) {
		return data;
	}

	@Override
	protected FreeTextDataPersist importExportMapDataToPersistInternal(FreeTextDataImportExport data, FreeTextDataPersist persist) {
		return persist;
	}

	@Override
	protected FreeTextDataImportExport dataToImportExportXmlInternal(FreeTextDataEntity data, FreeTextDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(FreeTextDataEntity data) {
		return false;
	}
}
