package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.JournalRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.JournalRepositoryDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.DataRepositoryDataBuilder;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.JournalRepositoryDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.DataRepositoryData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.JournalRepositoryData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.JournalRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class JournalRepositoryFieldDataHelperService extends BaseFieldDataHelperService<JournalRepositoryData, JournalRepositoryDataPersist, JournalRepositoryDataEntity, JournalRepositoryDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public JournalRepositoryFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.JOURNAL_REPOSITORIES;
	}

	@Override
	public JournalRepositoryDataEntity newDataInstanceInternal() {
		return new JournalRepositoryDataEntity();
	}

	@Override
	public JournalRepositoryData newModelInstanceInternal() {
		return new JournalRepositoryData();
	}

	@Override
	public JournalRepositoryDataPersist newPersistModelInstanceInternal() {
		return new JournalRepositoryDataPersist();
	}

	@Override
	protected JournalRepositoryDataImportExport newImportExportInstanceInternal() {
		return new JournalRepositoryDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return JournalRepositoryDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return JournalRepositoryData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return JournalRepositoryDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return JournalRepositoryDataImportExport.class;
	}

	@Override
	public List<JournalRepositoryData> buildInternal(FieldSet fieldSet, List<JournalRepositoryDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(JournalRepositoryDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected JournalRepositoryDataEntity applyPersistInternal(JournalRepositoryDataPersist persist, JournalRepositoryDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected JournalRepositoryDataPersist importExportMapDataToPersistInternal(JournalRepositoryDataImportExport data, JournalRepositoryDataPersist persist){
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected JournalRepositoryDataImportExport dataToImportExportXmlInternal(JournalRepositoryDataEntity data, JournalRepositoryDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(JournalRepositoryDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
