package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.LicenseDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.LicenseDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.LicenseDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.LicenseData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.LicenseDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class LicenseFieldDataHelperService extends BaseFieldDataHelperService<LicenseData, LicenseDataPersist, LicenseDataEntity, LicenseDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public LicenseFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.LICENSES;
	}

	@Override
	public LicenseDataEntity newDataInstanceInternal() {
		return new LicenseDataEntity();
	}

	@Override
	public LicenseData newModelInstanceInternal() {
		return new LicenseData();
	}

	@Override
	public LicenseDataPersist newPersistModelInstanceInternal() {
		return new LicenseDataPersist();
	}

	@Override
	protected LicenseDataImportExport newImportExportInstanceInternal() {
		return new LicenseDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return LicenseDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return LicenseData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return LicenseDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return LicenseDataImportExport.class;
	}

	@Override
	public List<LicenseData> buildInternal(FieldSet fieldSet, List<LicenseDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(LicenseDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected LicenseDataEntity applyPersistInternal(LicenseDataPersist persist, LicenseDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected LicenseDataPersist importExportMapDataToPersistInternal(LicenseDataImportExport data, LicenseDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected LicenseDataImportExport dataToImportExportXmlInternal(LicenseDataEntity data, LicenseDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(LicenseDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
