package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.OrganizationDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.OrganizationDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.OrganizationDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.OrganizationData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.OrganizationDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class OrganizationFieldDataHelperService extends BaseFieldDataHelperService<OrganizationData, OrganizationDataPersist, OrganizationDataEntity, OrganizationDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public OrganizationFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.ORGANIZATIONS;
	}

	@Override
	public OrganizationDataEntity newDataInstanceInternal() {
		return new OrganizationDataEntity();
	}

	@Override
	public OrganizationData newModelInstanceInternal() {
		return new OrganizationData();
	}

	@Override
	public OrganizationDataPersist newPersistModelInstanceInternal() {
		return new OrganizationDataPersist();
	}

	@Override
	protected OrganizationDataImportExport newImportExportInstanceInternal() {
		return new OrganizationDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return OrganizationDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return OrganizationData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return OrganizationDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return OrganizationDataImportExport.class;
	}

	@Override
	public List<OrganizationData> buildInternal(FieldSet fieldSet, List<OrganizationDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(OrganizationDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected OrganizationDataEntity applyPersistInternal(OrganizationDataPersist persist, OrganizationDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected OrganizationDataPersist importExportMapDataToPersistInternal(OrganizationDataImportExport data, OrganizationDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected OrganizationDataImportExport dataToImportExportXmlInternal(OrganizationDataEntity data, OrganizationDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(OrganizationDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
