package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.PublicationDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.PublicationDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.PublicationDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.PublicationData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.PublicationDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class PublicationFieldDataHelperService extends BaseFieldDataHelperService<PublicationData, PublicationDataPersist, PublicationDataEntity, PublicationDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public PublicationFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.PUBLICATIONS;
	}

	@Override
	public PublicationDataEntity newDataInstanceInternal() {
		return new PublicationDataEntity();
	}

	@Override
	public PublicationData newModelInstanceInternal() {
		return new PublicationData();
	}

	@Override
	public PublicationDataPersist newPersistModelInstanceInternal() {
		return new PublicationDataPersist();
	}

	@Override
	protected PublicationDataImportExport newImportExportInstanceInternal() {
		return new PublicationDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return PublicationDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return PublicationData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return PublicationDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return PublicationDataImportExport.class;
	}

	@Override
	protected boolean isMultiValueInternal(PublicationDataEntity data) {
		return data.getMultiAutoComplete();
	}

	@Override
	public List<PublicationData> buildInternal(FieldSet fieldSet, List<PublicationDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(PublicationDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected PublicationDataEntity applyPersistInternal(PublicationDataPersist persist, PublicationDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected PublicationDataPersist importExportMapDataToPersistInternal(PublicationDataImportExport data, PublicationDataPersist persist){
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected PublicationDataImportExport dataToImportExportXmlInternal(PublicationDataEntity data, PublicationDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}
}
