package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.PublicationRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.PublicationRepositoryDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.DataRepositoryDataBuilder;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.PublicationRepositoryDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.DataRepositoryData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.PublicationRepositoryData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.PublicationRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class PublicationRepositoryFieldDataHelperService extends BaseFieldDataHelperService<PublicationRepositoryData, PublicationRepositoryDataPersist, PublicationRepositoryDataEntity, PublicationRepositoryDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public PublicationRepositoryFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.PUB_REPOSITORIES;
	}

	@Override
	public PublicationRepositoryDataEntity newDataInstanceInternal() {
		return new PublicationRepositoryDataEntity();
	}

	@Override
	public PublicationRepositoryData newModelInstanceInternal() {
		return new PublicationRepositoryData();
	}

	@Override
	public PublicationRepositoryDataPersist newPersistModelInstanceInternal() {
		return new PublicationRepositoryDataPersist();
	}

	@Override
	protected PublicationRepositoryDataImportExport newImportExportInstanceInternal() {
		return new PublicationRepositoryDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return PublicationRepositoryDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return PublicationRepositoryData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return PublicationRepositoryDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return PublicationRepositoryDataImportExport.class;
	}

	@Override
	public List<PublicationRepositoryData> buildInternal(FieldSet fieldSet, List<PublicationRepositoryDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(PublicationRepositoryDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected PublicationRepositoryDataEntity applyPersistInternal(PublicationRepositoryDataPersist persist, PublicationRepositoryDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected PublicationRepositoryDataPersist importExportMapDataToPersistInternal(PublicationRepositoryDataImportExport data, PublicationRepositoryDataPersist persist){
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected PublicationRepositoryDataImportExport dataToImportExportXmlInternal(PublicationRepositoryDataEntity data, PublicationRepositoryDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(PublicationRepositoryDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
