package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.AutoCompleteDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.RadioBoxDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.UploadDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.RadioBoxDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.RadioBoxOption;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.UploadDataOption;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.RadioBoxDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.RadioBoxData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class RadioBoxFieldDataHelperService extends BaseFieldDataHelperService<RadioBoxData, RadioBoxDataPersist, RadioBoxDataEntity, RadioBoxDataImportExport> {

	private final ConventionService conventionService;
	private final BuilderFactory builderFactory;

	public RadioBoxFieldDataHelperService(ConventionService conventionService, BuilderFactory builderFactory) {
		this.conventionService = conventionService;
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.RADIO_BOX;
	}

	@Override
	public RadioBoxDataEntity newDataInstanceInternal() {
		return new RadioBoxDataEntity();
	}

	@Override
	public RadioBoxData newModelInstanceInternal() {
		return new RadioBoxData();
	}

	@Override
	public RadioBoxDataPersist newPersistModelInstanceInternal() {
		return new RadioBoxDataPersist();
	}

	@Override
	protected RadioBoxDataImportExport newImportExportInstanceInternal() {
		return new RadioBoxDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return RadioBoxDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return RadioBoxData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return RadioBoxDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return RadioBoxDataImportExport.class;
	}

	@Override
	public List<RadioBoxData> buildInternal(FieldSet fieldSet, List<RadioBoxDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(RadioBoxDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected RadioBoxDataEntity applyPersistInternal(RadioBoxDataPersist persist, RadioBoxDataEntity data) {
		if (!this.conventionService.isListNullOrEmpty(persist.getOptions())){
			data.setOptions(new ArrayList<>());
			for (RadioBoxOptionPersist radioBoxOptionPersist: persist.getOptions()) {
				data.getOptions().add(this.buildOption(radioBoxOptionPersist));
			}
		}
		return data;
	}

	private @NotNull RadioBoxDataEntity.Option buildOption(RadioBoxOptionPersist persist){
		RadioBoxDataEntity.Option data = new RadioBoxDataEntity.Option();
		if (persist == null) return data;

		data.setLabel(persist.getLabel());
		data.setValue(persist.getValue());

		return data;
	}

	@Override
	protected RadioBoxDataPersist importExportMapDataToPersistInternal(RadioBoxDataImportExport data, RadioBoxDataPersist persist){
		if (!this.conventionService.isListNullOrEmpty(data.getOptions())){
			persist.setOptions(new ArrayList<>());
			for (RadioBoxOption radioBoxOption: data.getOptions()) {
				persist.getOptions().add(this.buildOption(radioBoxOption));
			}
		}
		return persist;
	}

	private @NotNull RadioBoxOptionPersist buildOption(RadioBoxOption data){
		RadioBoxOptionPersist persist = new RadioBoxOptionPersist();
		if (data == null) return persist;

		persist.setLabel(data.getLabel());
		persist.setValue(data.getValue());

		return persist;
	}

	@Override
	protected RadioBoxDataImportExport dataToImportExportXmlInternal(RadioBoxDataEntity data, RadioBoxDataImportExport xml) {
		if (!this.conventionService.isListNullOrEmpty(data.getOptions())){
			xml.setOptions(new ArrayList<>());
			for (RadioBoxDataEntity.Option radioBoxOption: data.getOptions()) {
				xml.getOptions().add(this.buildOption(radioBoxOption));
			}
		}
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(RadioBoxDataEntity data) {
		return false;
	}

	private @NotNull RadioBoxOption buildOption(RadioBoxDataEntity.Option data){
		RadioBoxOption xml = new RadioBoxOption();
		if (data == null) return xml;

		xml.setLabel(data.getLabel());
		xml.setValue(data.getValue());

		return xml;
	}

}
