package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.RegistryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.RegistryDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.RegistryDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.RegistryData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.RegistryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class RegistryFieldDataHelperService extends BaseFieldDataHelperService<RegistryData, RegistryDataPersist, RegistryDataEntity, RegistryDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public RegistryFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.REGISTRIES;
	}

	@Override
	public RegistryDataEntity newDataInstanceInternal() {
		return new RegistryDataEntity();
	}

	@Override
	public RegistryData newModelInstanceInternal() {
		return new RegistryData();
	}

	@Override
	public RegistryDataPersist newPersistModelInstanceInternal() {
		return new RegistryDataPersist();
	}

	@Override
	protected RegistryDataImportExport newImportExportInstanceInternal() {
		return new RegistryDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return RegistryDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return RegistryData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return RegistryDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return RegistryDataImportExport.class;
	}

	@Override
	public List<RegistryData> buildInternal(FieldSet fieldSet, List<RegistryDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(RegistryDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected RegistryDataEntity applyPersistInternal(RegistryDataPersist persist, RegistryDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected RegistryDataPersist importExportMapDataToPersistInternal(RegistryDataImportExport data, RegistryDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected RegistryDataImportExport dataToImportExportXmlInternal(RegistryDataEntity data, RegistryDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(RegistryDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
