package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ResearcherDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ResearcherDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ResearcherDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ResearcherData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.ResearcherDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class ResearcherFieldDataHelperService extends BaseFieldDataHelperService<ResearcherData, ResearcherDataPersist, ResearcherDataEntity, ResearcherDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public ResearcherFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.RESEARCHERS;
	}

	@Override
	public ResearcherDataEntity newDataInstanceInternal() {
		return new ResearcherDataEntity();
	}

	@Override
	public ResearcherData newModelInstanceInternal() {
		return new ResearcherData();
	}

	@Override
	public ResearcherDataPersist newPersistModelInstanceInternal() {
		return new ResearcherDataPersist();
	}

	@Override
	protected ResearcherDataImportExport newImportExportInstanceInternal() {
		return new ResearcherDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ResearcherDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ResearcherData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ResearcherDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ResearcherDataImportExport.class;
	}

	@Override
	public List<ResearcherData> buildInternal(FieldSet fieldSet, List<ResearcherDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ResearcherDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ResearcherDataEntity applyPersistInternal(ResearcherDataPersist persist, ResearcherDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected ResearcherDataPersist importExportMapDataToPersistInternal(ResearcherDataImportExport data, ResearcherDataPersist persist){
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected ResearcherDataImportExport dataToImportExportXmlInternal(ResearcherDataEntity data, ResearcherDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ResearcherDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
