package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BooleanDecisionDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.RichTextAreaDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.RichTextAreaDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.RichTextAreaDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.RichTextAreaData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BooleanDecisionDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.RichTextAreaDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class RichTextAreaDataFieldDataHelperService extends BaseFieldDataHelperService<RichTextAreaData, RichTextAreaDataPersist, RichTextAreaDataEntity, RichTextAreaDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public RichTextAreaDataFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.RICH_TEXT_AREA;
	}

	@Override
	public RichTextAreaDataEntity newDataInstanceInternal() {
		return new RichTextAreaDataEntity();
	}

	@Override
	public RichTextAreaData newModelInstanceInternal() {
		return new RichTextAreaData();
	}

	@Override
	public RichTextAreaDataPersist newPersistModelInstanceInternal() {
		return new RichTextAreaDataPersist();
	}

	@Override
	protected RichTextAreaDataImportExport newImportExportInstanceInternal() {
		return new RichTextAreaDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return RichTextAreaDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return RichTextAreaData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return RichTextAreaDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return RichTextAreaDataImportExport.class;
	}

	@Override
	public List<RichTextAreaData> buildInternal(FieldSet fieldSet, List<RichTextAreaDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(RichTextAreaDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected RichTextAreaDataEntity applyPersistInternal(RichTextAreaDataPersist persist, RichTextAreaDataEntity data) {
		return data;
	}

	@Override
	protected RichTextAreaDataPersist importExportMapDataToPersistInternal(RichTextAreaDataImportExport data, RichTextAreaDataPersist persist) {
		return persist;
	}

	@Override
	protected RichTextAreaDataImportExport dataToImportExportXmlInternal(RichTextAreaDataEntity data, RichTextAreaDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(RichTextAreaDataEntity data) {
		return false;
	}
}
