package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ServiceDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ServiceDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ServiceDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ServiceData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.ServiceDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class ServiceFieldDataHelperService extends BaseFieldDataHelperService<ServiceData, ServiceDataPersist, ServiceDataEntity, ServiceDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public ServiceFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.SERVICES;
	}

	@Override
	public ServiceDataEntity newDataInstanceInternal() {
		return new ServiceDataEntity();
	}

	@Override
	public ServiceData newModelInstanceInternal() {
		return new ServiceData();
	}

	@Override
	public ServiceDataPersist newPersistModelInstanceInternal() {
		return new ServiceDataPersist();
	}

	@Override
	protected ServiceDataImportExport newImportExportInstanceInternal() {
		return new ServiceDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ServiceDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ServiceData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ServiceDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ServiceDataImportExport.class;
	}

	@Override
	public List<ServiceData> buildInternal(FieldSet fieldSet, List<ServiceDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ServiceDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ServiceDataEntity applyPersistInternal(ServiceDataPersist persist, ServiceDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected ServiceDataPersist importExportMapDataToPersistInternal(ServiceDataImportExport data, ServiceDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected ServiceDataImportExport dataToImportExportXmlInternal(ServiceDataEntity data, ServiceDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ServiceDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
