package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BooleanDecisionDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.TagDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.TagDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.TagDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.TagData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BooleanDecisionDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.TagDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class TagFieldDataHelperService extends BaseFieldDataHelperService<TagData, TagDataPersist, TagDataEntity, TagDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public TagFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.TAGS;
	}

	@Override
	public TagDataEntity newDataInstanceInternal() {
		return new TagDataEntity();
	}

	@Override
	public TagDataPersist newPersistModelInstanceInternal() {
		return new TagDataPersist();
	}

	@Override
	protected TagDataImportExport newImportExportInstanceInternal() {
		return new TagDataImportExport();
	}

	@Override
	public TagData newModelInstanceInternal() {
		return new TagData();
	}

	@Override
	public Class<?> getDataClass() {
		return TagDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return TagData.class;
	}

	@Override
	public Class<?> getPersistModelClass() {
		return TagDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return TagDataImportExport.class;
	}

	@Override
	public List<TagData> buildInternal(FieldSet fieldSet, List<TagDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(TagDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected TagDataEntity applyPersistInternal(TagDataPersist persist, TagDataEntity data) {
		return data;
	}

	@Override
	protected TagDataPersist importExportMapDataToPersistInternal(TagDataImportExport data, TagDataPersist persist) {
		return persist;
	}

	@Override
	protected TagDataImportExport dataToImportExportXmlInternal(TagDataEntity data, TagDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(TagDataEntity data) {
		return false;
	}
}
