package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.TaxonomyDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.TaxonomyDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.TaxonomyDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.TaxonomyData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.TaxonomyDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class TaxonomyFieldDataHelperService extends BaseFieldDataHelperService<TaxonomyData, TaxonomyDataPersist, TaxonomyDataEntity, TaxonomyDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public TaxonomyFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.TAXONOMIES;
	}

	@Override
	public TaxonomyDataEntity newDataInstanceInternal() {
		return new TaxonomyDataEntity();
	}

	@Override
	public TaxonomyData newModelInstanceInternal() {
		return new TaxonomyData();
	}

	@Override
	public TaxonomyDataPersist newPersistModelInstanceInternal() {
		return new TaxonomyDataPersist();
	}

	@Override
	protected TaxonomyDataImportExport newImportExportInstanceInternal() {
		return new TaxonomyDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return TaxonomyDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return TaxonomyData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return TaxonomyDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return TaxonomyDataImportExport.class;
	}

	@Override
	public List<TaxonomyData> buildInternal(FieldSet fieldSet, List<TaxonomyDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(TaxonomyDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected TaxonomyDataEntity applyPersistInternal(TaxonomyDataPersist persist, TaxonomyDataEntity data) {
		data.setMultiAutoComplete(persist.getMultiAutoComplete());
		return data;
	}

	@Override
	protected TaxonomyDataPersist importExportMapDataToPersistInternal(TaxonomyDataImportExport data, TaxonomyDataPersist persist) {
		persist.setMultiAutoComplete(data.getMultiAutoComplete());
		return persist;
	}

	@Override
	protected TaxonomyDataImportExport dataToImportExportXmlInternal(TaxonomyDataEntity data, TaxonomyDataImportExport xml) {
		xml.setMultiAutoComplete(data.getMultiAutoComplete());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(TaxonomyDataEntity data) {
		return data.getMultiAutoComplete();
	}
}
