package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.DataRepositoryDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.TextAreaDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.TextAreaDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.TextAreaDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.TextAreaData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.BaseFieldDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.DataRepositoryDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.TextAreaDataPersist;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.WordListDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class TextAreaFieldDataHelperService extends BaseFieldDataHelperService<TextAreaData, TextAreaDataPersist, TextAreaDataEntity, TextAreaDataImportExport> {
	
	private final BuilderFactory builderFactory;

	public TextAreaFieldDataHelperService(BuilderFactory builderFactory) {
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.TEXT_AREA;
	}

	@Override
	public TextAreaDataEntity newDataInstanceInternal() {
		return new TextAreaDataEntity();
	}

	@Override
	public TextAreaData newModelInstanceInternal() {
		return new TextAreaData();
	}

	@Override
	public TextAreaDataPersist newPersistModelInstanceInternal() {
		return new TextAreaDataPersist();
	}

	@Override
	protected TextAreaDataImportExport newImportExportInstanceInternal() {
		return new TextAreaDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return TextAreaDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return TextAreaData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return TextAreaDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return TextAreaDataImportExport.class;
	}

	@Override
	public List<TextAreaData> buildInternal(FieldSet fieldSet, List<TextAreaDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(TextAreaDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected TextAreaDataEntity applyPersistInternal(TextAreaDataPersist persist, TextAreaDataEntity data) {
		return data;
	}

	@Override
	protected TextAreaDataPersist importExportMapDataToPersistInternal(TextAreaDataImportExport data, TextAreaDataPersist persist) {
		return persist;
	}

	@Override
	protected TextAreaDataImportExport dataToImportExportXmlInternal(TextAreaDataEntity data, TextAreaDataImportExport xml) {
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(TextAreaDataEntity data) {
		return false;
	}
}
