package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.BaseFieldDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.RadioBoxDataEntity;
import eu.eudat.commons.types.descriptiontemplate.fielddata.UploadDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.UploadDataImportExport;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.UploadDataOption;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.UploadDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.UploadData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class UploadFieldDataHelperService extends BaseFieldDataHelperService<UploadData, UploadDataPersist, UploadDataEntity, UploadDataImportExport> {

	private final ConventionService conventionService;
	private final BuilderFactory builderFactory;

	public UploadFieldDataHelperService(ConventionService conventionService, BuilderFactory builderFactory) {
		this.conventionService = conventionService;
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.UPLOAD;
	}

	@Override
	public UploadDataEntity newDataInstanceInternal() {
		return new UploadDataEntity();
	}

	@Override
	public UploadData newModelInstanceInternal() {
		return new UploadData();
	}

	@Override
	public UploadDataPersist newPersistModelInstanceInternal() {
		return new UploadDataPersist();
	}

	@Override
	protected UploadDataImportExport newImportExportInstanceInternal() {
		return new UploadDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return UploadDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return UploadData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return UploadDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return UploadDataImportExport.class;
	}

	@Override
	public List<UploadData> buildInternal(FieldSet fieldSet, List<UploadDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(UploadDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected UploadDataEntity applyPersistInternal(UploadDataPersist persist, UploadDataEntity data) {
		if (!this.conventionService.isListNullOrEmpty(persist.getTypes())){
			data.setTypes(new ArrayList<>());
			for (UploadOptionPersist uploadOptionPersist: persist.getTypes()) {
				data.getTypes().add(this.buildOption(uploadOptionPersist));
			}
		}
		data.setMaxFileSizeInMB(persist.getMaxFileSizeInMB());
		return data;
	}

	private @NotNull UploadDataEntity.Option buildOption(UploadOptionPersist persist){
		UploadDataEntity.Option data = new UploadDataEntity.Option();
		if (persist == null) return data;

		data.setLabel(persist.getLabel());
		data.setValue(persist.getValue());

		return data;
	}

	@Override
	protected UploadDataPersist importExportMapDataToPersistInternal(UploadDataImportExport data, UploadDataPersist persist) {
		if (!this.conventionService.isListNullOrEmpty(data.getTypes())){
			persist.setTypes(new ArrayList<>());
			for (UploadDataOption option: data.getTypes()) {
				persist.getTypes().add(this.buildOption(option));
			}
		}
		persist.setMaxFileSizeInMB(data.getMaxFileSizeInMB());
		return persist;
	}

	private @NotNull UploadOptionPersist buildOption(UploadDataOption data){
		UploadOptionPersist persist = new UploadOptionPersist();
		if (data == null) return persist;

		persist.setLabel(data.getLabel());
		persist.setValue(data.getValue());

		return persist;
	}

	@Override
	protected UploadDataImportExport dataToImportExportXmlInternal(UploadDataEntity data, UploadDataImportExport xml) {
		if (!this.conventionService.isListNullOrEmpty(data.getTypes())){
			xml.setTypes(new ArrayList<>());
			for (UploadDataEntity.Option option: data.getTypes()) {
				xml.getTypes().add(this.buildOption(option));
			}
		}
		xml.setMaxFileSizeInMB(data.getMaxFileSizeInMB());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(UploadDataEntity data) {
		return false;
	}

	private @NotNull UploadDataOption buildOption(UploadDataEntity.Option data){
		UploadDataOption xml = new UploadDataOption();
		if (data == null) return xml;

		xml.setLabel(data.getLabel());
		xml.setValue(data.getValue());

		return xml;
	}
}
