package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.enums.FieldType;
import eu.eudat.commons.types.descriptiontemplate.fielddata.*;
import eu.eudat.commons.types.descriptiontemplate.fielddata.WordListDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ComboBoxDataOptionEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.WordListDataImportExport;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.WordListDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.BaseFieldData;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.WordListData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class WordListFieldDataHelperService extends BaseFieldDataHelperService<WordListData, WordListDataPersist, WordListDataEntity, WordListDataImportExport> {

	private final ConventionService conventionService;
	private final BuilderFactory builderFactory;

	public WordListFieldDataHelperService(ConventionService conventionService, BuilderFactory builderFactory) {
		this.conventionService = conventionService;
		this.builderFactory = builderFactory;
	}


	@Override
	public FieldType getFormType() {
		return FieldType.WORD_LIST;
	}

	@Override
	public WordListDataEntity newDataInstanceInternal() {
		return new WordListDataEntity();
	}

	@Override
	public WordListData newModelInstanceInternal() {
		return new WordListData();
	}

	@Override
	public WordListDataPersist newPersistModelInstanceInternal() {
		return new WordListDataPersist();
	}

	@Override
	protected WordListDataImportExport newImportExportInstanceInternal() {
		return new WordListDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return WordListDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return WordListData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return WordListDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return WordListDataImportExport.class;
	}

	@Override
	public List<WordListData> buildInternal(FieldSet fieldSet, List<WordListDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(WordListDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected WordListDataEntity applyPersistInternal(WordListDataPersist persist, WordListDataEntity data) {
		data.setMultiList(persist.getMultiList());
		if (!this.conventionService.isListNullOrEmpty(persist.getOptions())){
			data.setOptions(new ArrayList<>());
			for (ComboBoxOptionPersist optionPersist: persist.getOptions()) {
				data.getOptions().add(this.buildOption(optionPersist));
			}
		}
		return data;
	}


	private @NotNull ComboBoxDataEntity.Option buildOption(ComboBoxOptionPersist persist){
		ComboBoxDataEntity.Option data = new ComboBoxDataEntity.Option();
		if (persist == null) return data;

		data.setLabel(persist.getLabel());
		data.setValue(persist.getValue());
		data.setUri(persist.getUri());
		data.setSource(persist.getSource());

		return data;
	}

	@Override
	protected WordListDataPersist importExportMapDataToPersistInternal(WordListDataImportExport data, WordListDataPersist persist) {
		persist.setMultiList(data.getMultiList());
		if (!this.conventionService.isListNullOrEmpty(data.getOptions())){
			persist.setOptions(new ArrayList<>());
			for (ComboBoxDataOptionEntity option: data.getOptions()) {
				persist.getOptions().add(this.buildOption(option));
			}
		}
		return persist;
	}

	private @NotNull ComboBoxOptionPersist buildOption(ComboBoxDataOptionEntity data){
		ComboBoxOptionPersist persist = new ComboBoxOptionPersist();
		if (data == null) return persist;

		persist.setLabel(data.getLabel());
		persist.setValue(data.getValue());
		persist.setUri(data.getUri());
		persist.setSource(data.getSource());

		return persist;
	}

	@Override
	protected WordListDataImportExport dataToImportExportXmlInternal(WordListDataEntity data, WordListDataImportExport xml) {
		xml.setMultiList(data.getMultiList());
		if (!this.conventionService.isListNullOrEmpty(data.getOptions())){
			xml.setOptions(new ArrayList<>());
			for (ComboBoxDataEntity.Option option: data.getOptions()) {
				xml.getOptions().add(this.buildOption(option));
			}
		}
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(WordListDataEntity data) {
		return data.getMultiList();
	}

	private @NotNull ComboBoxDataOptionEntity buildOption(ComboBoxDataEntity.Option data){
		ComboBoxDataOptionEntity xml = new ComboBoxDataOptionEntity();
		if (data == null) return xml;

		xml.setLabel(data.getLabel());
		xml.setValue(data.getValue());
		xml.setUri(data.getUri());
		xml.setSource(data.getSource());

		return xml;
	}

}
