package eu.eudat.controllers;

import eu.eudat.authorization.Permission;
import eu.eudat.exceptions.emailconfirmation.HasConfirmedEmailException;
import eu.eudat.exceptions.emailconfirmation.TokenExpiredException;
import eu.eudat.logic.managers.UnlinkEmailConfirmationManager;
import eu.eudat.models.data.helpers.responses.ResponseItem;
import eu.eudat.types.ApiMessageCode;
import gr.cite.commons.web.authz.service.AuthorizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import jakarta.transaction.Transactional;

import javax.management.InvalidApplicationException;

@RestController
@CrossOrigin
@RequestMapping(value = "api/emailUnlinkConfirmation")
public class EmailUnlinkConfirmation {

    private UnlinkEmailConfirmationManager unlinkEmailConfirmationManager;
    private final AuthorizationService authorizationService;

    @Autowired
    public EmailUnlinkConfirmation(UnlinkEmailConfirmationManager unlinkEmailConfirmationManager, AuthorizationService authorizationService){
        this.unlinkEmailConfirmationManager = unlinkEmailConfirmationManager;
        this.authorizationService = authorizationService;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.GET, value = {"/{emailToken}"})
    public @ResponseBody
    ResponseEntity<ResponseItem> emailConfirmation(@PathVariable(value = "emailToken") String token) {
        try {
            this.unlinkEmailConfirmationManager.confirmEmail(token);
            return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem().status(ApiMessageCode.SUCCESS_MESSAGE));
        } catch (TokenExpiredException | HasConfirmedEmailException | InvalidApplicationException ex) {
            if (ex instanceof TokenExpiredException) {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ResponseItem().status(ApiMessageCode.NO_MESSAGE));
            }
            else {
                return ResponseEntity.status(HttpStatus.FOUND).body(new ResponseItem().status(ApiMessageCode.WARN_MESSAGE));
            }
        }
    }

//    @Transactional
//    @RequestMapping(method = RequestMethod.POST, consumes = "application/json", produces = "application/json")
//    public @ResponseBody
//    ResponseEntity<ResponseItem> sendUnlinkConfirmationEmail(@RequestBody UserUnlinkRequestModel requestModel) {
//        this.authorizationService.authorizeForce(Permission.AuthenticatedRole);
//        try {
//            this.unlinkEmailConfirmationManager.sendConfirmationEmail(requestModel.getEmail(), requestModel.getUserId(), requestModel.getProvider());
//            return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem().status(ApiMessageCode.SUCCESS_MESSAGE));
//        } catch (Exception ex) {
//            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ResponseItem().status(ApiMessageCode.NO_MESSAGE).message("Could not send unlink email."));
//        }
//    }
}
