package eu.eudat.controllers;

import eu.eudat.authorization.Permission;
import eu.eudat.logic.managers.InvitationsManager;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.models.data.helpers.responses.ResponseItem;
import eu.eudat.models.data.invitation.Invitation;
import eu.eudat.types.ApiMessageCode;
import gr.cite.commons.web.authz.service.AuthorizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import jakarta.xml.bind.JAXBException;

import javax.management.InvalidApplicationException;
import java.io.IOException;
import java.util.List;
import java.util.UUID;


@RequestMapping("api/invite/")
@RestController
@CrossOrigin
public class UserInvitationController extends BaseController {

    private InvitationsManager invitationsManager;
    private final AuthorizationService authorizationService;
    @Autowired
    public UserInvitationController(ApiContext apiContext, InvitationsManager invitationsManager, AuthorizationService authorizationService) {
        super(apiContext);
        this.invitationsManager = invitationsManager;
        this.authorizationService = authorizationService;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST, value = {"/users"}, consumes = "application/json", produces = "application/json")
    public @ResponseBody
    ResponseEntity<ResponseItem<Invitation>> users(@RequestBody Invitation invitation) throws Exception {
        this.authorizationService.authorizeForce(Permission.AuthenticatedRole);
        
        this.invitationsManager.inviteUsers(invitation);
        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<Invitation>().status(ApiMessageCode.SUCCESS_MESSAGE).message("Users have beeen invited"));
    }

    @Transactional
    @RequestMapping(method = RequestMethod.GET, value = {"/exchange/{invitationID}"}, produces = "application/json")
    public @ResponseBody
    ResponseEntity<ResponseItem<UUID>> exchange(@PathVariable UUID invitationID) throws JAXBException, IOException, InvalidApplicationException {
        this.authorizationService.authorizeForce(Permission.AuthenticatedRole);

        UUID dmpId = invitationsManager.assignUserAcceptedInvitation(invitationID);
        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<UUID>().status(ApiMessageCode.SUCCESS_MESSAGE).payload(dmpId));
    }

//    @RequestMapping(method = RequestMethod.POST, value = {"/getUsers"}, consumes = "application/json", produces = "application/json")
//    public @ResponseBody
////    ResponseEntity<ResponseItem<List<UserInfoInvitationModel>>> getUsers() throws IllegalAccessException, InstantiationException {
//    ResponseEntity<ResponseItem<List<UserInfoInvitationModel>>> getUsers(@RequestBody UserInfoRequestItem userInfoRequestItem) throws IllegalAccessException, InstantiationException, InvalidApplicationException {
////        List<UserInfoInvitationModel> users = invitationsManager.getUsers(principal);
//        this.authorizationService.authorizeForce(Permission.AuthenticatedRole);
//
//        List<UserInfoInvitationModel> users = invitationsManager.getUsersWithCriteria(userInfoRequestItem);
//        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<List<UserInfoInvitationModel>>().status(ApiMessageCode.SUCCESS_MESSAGE).payload(users));
//    }
}
