package eu.eudat.controllers.v2;

import eu.eudat.audit.AuditableAction;
import eu.eudat.commons.scope.user.UserScope;
import eu.eudat.commons.validation.ValidationFilterAnnotation;
import eu.eudat.model.DashboardStatistics;
import eu.eudat.model.RecentActivityItem;
import eu.eudat.model.RecentActivityItemLookup;
import eu.eudat.model.User;
import eu.eudat.model.censorship.MyDashboardStatisticsCensor;
import eu.eudat.model.censorship.PublicDashboardStatisticsCensor;
import eu.eudat.model.censorship.RecentActivityItemCensor;
import eu.eudat.model.persist.ContactSupportPersist;
import eu.eudat.model.persist.DescriptionTemplateTypePersist;
import eu.eudat.model.persist.PublicContactSupportPersist;
import eu.eudat.service.contactsupport.ContactSupportService;
import eu.eudat.service.dashborad.DashboardService;
import gr.cite.tools.auditing.AuditService;
import gr.cite.tools.data.censor.CensorFactory;
import gr.cite.tools.exception.MyApplicationException;
import gr.cite.tools.exception.MyForbiddenException;
import gr.cite.tools.exception.MyNotFoundException;
import gr.cite.tools.logging.LoggerService;
import gr.cite.tools.logging.MapLogEntry;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.management.InvalidApplicationException;
import java.util.AbstractMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping(path = "api/contact-support")
public class ContactSupportController {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(ContactSupportController.class));

    private final AuditService auditService;

    private final ContactSupportService contactSupportService;


    public ContactSupportController(
            AuditService auditService,
            ContactSupportService contactSupportService) {
        this.auditService = auditService;
        this.contactSupportService = contactSupportService;
    }



    @PostMapping("send")
    @Transactional
    @ValidationFilterAnnotation(validator = ContactSupportPersist.ContactSupportPersistValidator.ValidatorName, argumentName ="model")
    public void sendContactEmail(@RequestBody ContactSupportPersist model) throws InvalidApplicationException {
        logger.debug(new MapLogEntry("send support email").And("model", model));

        this.contactSupportService.sendContactEmail(model);
        
        this.auditService.track(AuditableAction.ContactSupport_Sent, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("model", model)
        ));
        //this.auditService.trackIdentity(AuditableAction.IdentityTracking_Action);
    }

    

    @PostMapping("public/send")
    @Transactional
    @ValidationFilterAnnotation(validator = PublicContactSupportPersist.PublicContactSupportPersistValidator.ValidatorName, argumentName ="model")
    public void sendPublicContactEmail(@RequestBody PublicContactSupportPersist model) throws InvalidApplicationException {
        logger.debug(new MapLogEntry("send public support email").And("model", model));

        this.contactSupportService.sendPublicContactEmail(model);

        this.auditService.track(AuditableAction.ContactSupport_PublicSent, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("model", model)
        ));
        //this.auditService.trackIdentity(AuditableAction.IdentityTracking_Action);
    }

}
