package eu.eudat.controllers.v2;

import eu.eudat.audit.AuditableAction;
import eu.eudat.controllers.BaseController;
import eu.eudat.file.transformer.models.misc.FileFormat;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.model.file.ExportRequestModel;
import eu.eudat.model.file.FileEnvelope;
import eu.eudat.service.transformer.FileTransformerService;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.auditing.AuditService;
import gr.cite.tools.data.censor.CensorFactory;
import gr.cite.tools.logging.LoggerService;
import gr.cite.tools.logging.MapLogEntry;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.management.InvalidApplicationException;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;
import java.util.UUID;

@RestController
@CrossOrigin
@RequestMapping(value = {"/api/file-transformer/"})
public class FileTransformerController extends BaseController {
    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(FileTransformerController.class));


    private final AuthorizationService authorizationService;
    private final FileTransformerService fileTransformerService;
    private final CensorFactory censorFactory;
    private final AuditService auditService;

    @Autowired
    public FileTransformerController(ApiContext apiContext, AuthorizationService authorizationService, FileTransformerService fileTransformerService, CensorFactory censorFactory, AuditService auditService){
        super(apiContext);
        this.authorizationService = authorizationService;
        this.fileTransformerService = fileTransformerService;
	    this.censorFactory = censorFactory;
	    this.auditService = auditService;
    }

    @GetMapping("/available")
    public List<FileFormat> getAvailableConfigurations() {
        logger.debug(new MapLogEntry("getAvailableConfigurations"));

        List<FileFormat> model = this.fileTransformerService.getAvailableConfigurations();
        this.auditService.track(AuditableAction.FileTransformer_GetAvailableConfigurations);
        //this.auditService.trackIdentity(AuditableAction.IdentityTracking_Action);

        return model;
    }

    @PostMapping("/export-dmp")
    public ResponseEntity<byte[]> export(@RequestBody ExportRequestModel requestModel) throws InvalidApplicationException, IOException {
        logger.debug(new MapLogEntry("exporting dmp"));
        HttpHeaders headers = new HttpHeaders();

        FileEnvelope fileEnvelope = this.fileTransformerService.exportDmp(requestModel.getId(), requestModel.getFormat());
        headers.add("Content-Disposition", "attachment;filename=" + fileEnvelope.getFilename());
        byte[] data = Files.readAllBytes(fileEnvelope.getFile().toPath());
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        return new ResponseEntity<>(data, headers, HttpStatus.OK);
    }

    @PostMapping("/export-description")
    public ResponseEntity<byte[]> exportDescription(@RequestBody ExportRequestModel requestModel) throws InvalidApplicationException, IOException {
        logger.debug(new MapLogEntry("exporting dmp"));
        HttpHeaders headers = new HttpHeaders();

        FileEnvelope fileEnvelope = this.fileTransformerService.exportDescription(requestModel.getId(), requestModel.getFormat());
        headers.add("Content-Disposition", "attachment;filename=" + fileEnvelope.getFilename());
        byte[] data = Files.readAllBytes(fileEnvelope.getFile().toPath());
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        return new ResponseEntity<>(data, headers, HttpStatus.OK);
    }
}
