package eu.eudat.controllers.v2;

import eu.eudat.authorization.Permission;
import eu.eudat.data.DescriptionEntity;
import eu.eudat.models.data.helpers.responses.ResponseItem;
import eu.eudat.service.maintance.MaintenanceService;
import eu.eudat.types.ApiMessageCode;
import gr.cite.commons.web.authz.service.AuthorizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "api/maintenance")
public class MaintenanceController {

    private final AuthorizationService authorizationService;
    private final MaintenanceService maintenanceService;

    @Autowired
    public MaintenanceController(AuthorizationService authorizationService, MaintenanceService maintenanceService) {
        this.authorizationService = authorizationService;
        this.maintenanceService = maintenanceService;
    }

    /*
     * Data Index
     * */

    @RequestMapping(method = RequestMethod.POST, value = {"/index/elastic"})
    public @ResponseBody
    ResponseEntity<ResponseItem<DescriptionEntity>> generateIndex() throws Exception {
        this.authorizationService.authorizeForce(Permission.ManageElastic);

        this.maintenanceService.generateElasticIndex();
        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<DescriptionEntity>().status(ApiMessageCode.SUCCESS_MESSAGE).message("Generated").payload(null));
    }

    @RequestMapping(method = RequestMethod.DELETE, value = {"/index/elastic"})
    public @ResponseBody
    ResponseEntity<ResponseItem<DescriptionEntity>> clearIndex() throws Exception {
        this.authorizationService.authorizeForce(Permission.ManageElastic);

        this.maintenanceService.clearElasticIndex();
        return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<DescriptionEntity>().status(ApiMessageCode.SUCCESS_MESSAGE).message("Cleared").payload(null));
    }
}
