package eu.eudat.logic.managers;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import eu.eudat.commons.scope.user.UserScope;
import eu.eudat.data.old.EmailConfirmation;
import eu.eudat.data.UserEntity;
import eu.eudat.exceptions.emailconfirmation.HasConfirmedEmailException;
import eu.eudat.exceptions.emailconfirmation.TokenExpiredException;
import eu.eudat.logic.services.ApiContext;
import eu.eudat.logic.services.operations.DatabaseRepository;
import eu.eudat.model.UserContactInfo;
import eu.eudat.query.UserContactInfoQuery;
import eu.eudat.query.UserQuery;
import gr.cite.tools.data.query.Ordering;
import gr.cite.tools.data.query.QueryFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import jakarta.transaction.Transactional;

import javax.management.InvalidApplicationException;
import java.util.*;

@Component
public class UnlinkEmailConfirmationManager {

    private static Logger logger = LoggerFactory.getLogger(UnlinkEmailConfirmationManager.class);

    private ApiContext apiContext;
    private DatabaseRepository databaseRepository;
    private final QueryFactory queryFactory;
    private final UserScope userScope;

    @Autowired
    public UnlinkEmailConfirmationManager(ApiContext apiContext, QueryFactory queryFactory, UserScope userScope) {
        this.apiContext = apiContext;
        this.databaseRepository = apiContext.getOperationsContext().getDatabaseRepository();
        this.queryFactory = queryFactory;
        this.userScope = userScope;
    }

    @Transactional
    public void confirmEmail(String token) throws TokenExpiredException, HasConfirmedEmailException, InvalidApplicationException {
        EmailConfirmation loginConfirmationEmail = apiContext.getOperationsContext()
                .getDatabaseRepository().getLoginConfirmationEmailDao().asQueryable()
                .where((builder, root) -> builder.equal(root.get("token"), UUID.fromString(token))).getSingle();

        if (loginConfirmationEmail.getExpiresAt().compareTo(new Date()) < 0)
            throw new TokenExpiredException("Token has expired.");

        if(loginConfirmationEmail.getIsConfirmed())
            throw new HasConfirmedEmailException("Email is already confirmed.");

//        UserInfo userAskingForUnlink = databaseRepository.getUserInfoDao().asQueryable()
//                .where((builder, root) -> builder.equal(root.get("id"), loginConfirmationEmail.getUserId())).getSingle();

        try {
            Map<String, Object> map = new ObjectMapper().readValue(loginConfirmationEmail.getData(), new TypeReference<Map<String, Object>>() {});
            String emailTobeUnlinked = (String) map.get("email");
            Integer provider = Integer.valueOf((String) map.get("provider"));

            unlinkUser(emailTobeUnlinked, provider);

            loginConfirmationEmail.setIsConfirmed(true);
            databaseRepository.getLoginConfirmationEmailDao().createOrUpdate(loginConfirmationEmail);
        }
        catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    @Transactional
    private void unlinkUser(String emailTobeUnlinked, Integer provider) throws InvalidApplicationException {
//        UserCredentialEntity credential = databaseRepository.getCredentialDao().asQueryable()
//                .where((builder, root) -> builder.and(builder.equal(root.get("email"), emailTobeUnlinked), builder.equal(root.get("provider"), provider))).getSingle();
//        if(credential != null) {
//            databaseRepository.getCredentialDao().delete(credential);
//        }
    }

    public void sendUnlinkConfirmationEmail(String email, Integer provider) throws InvalidApplicationException {
        UserEntity user = this.queryFactory.query(UserQuery.class).ids(this.userScope.getUserId()).first();
        UserContactInfoQuery query = this.queryFactory.query(UserContactInfoQuery.class).userIds(user.getId());
        query.setOrder(new Ordering().addAscending(UserContactInfo._ordinal));
        if (query.first().getValue() != null && !query.first().getValue().equals(email)) {
            apiContext.getUtilitiesService().getConfirmationEmailService().sentUnlinkConfirmationEmail(
                    databaseRepository.getLoginConfirmationEmailDao(),
                    email,
                    user,
                    provider
            );
        }
    }
}
