package eu.eudat.logic.utilities.builders;

import eu.eudat.commons.types.common.DatabaseViewStyleDefinition;
import eu.eudat.commons.types.common.DatabaseModelDefinition;
import eu.eudat.logic.utilities.interfaces.ModelDefinition;
import eu.eudat.logic.utilities.interfaces.ViewStyleDefinition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.LinkedList;
import java.util.List;

public class ModelBuilder {
    private static final Logger logger = LoggerFactory.getLogger(ModelBuilder.class);
    public <U extends ModelDefinition<T>, T extends DatabaseModelDefinition> List<T> toModelDefinition(List<U> items, Class<T> clazz) {
        List<T> list = new LinkedList<T>();
        for (U item : items) {
            try {
                list.add(item.toDatabaseDefinition(clazz.newInstance()));
            } catch (InstantiationException e) {
                logger.error(e.getMessage(), e);
            } catch (IllegalAccessException e) {
                logger.error(e.getMessage(), e);
            }
        }
        return list;
    }

    public <U extends ViewStyleDefinition<T>, T extends DatabaseViewStyleDefinition> List<T> toViewStyleDefinition(List<U> items, Class<T> clazz) {
        List<T> list = new LinkedList<T>();
        for (U item : items) {
            try {
                list.add(item.toDatabaseDefinition(clazz.newInstance()));
            } catch (InstantiationException e) {
                logger.error(e.getMessage(), e);
            } catch (IllegalAccessException e) {
                logger.error(e.getMessage(), e);
            }
        }
        return list;
    }

    public <U extends ViewStyleDefinition<T>, T extends DatabaseViewStyleDefinition> List<U> fromViewStyleDefinition(List<T> items, Class<U> clazz) {
        List<U> list = new LinkedList<U>();
        for (T item : items) {
            try {
                U modelItem = clazz.newInstance();
                modelItem.fromDatabaseDefinition(item);
                list.add(modelItem);
            } catch (InstantiationException e) {
                logger.error(e.getMessage(), e);
            } catch (IllegalAccessException e) {
                logger.error(e.getMessage(), e);
            }
        }
        return list;
    }
}
