import { DescriptionTemplateFieldDataExternalDatasetType } from "@app/core/common/enum/description-template-field-data-external-dataset-type";
import { DescriptionTemplateFieldType } from "@app/core/common/enum/description-template-field-type";
import { DescriptionTemplateFieldValidationType } from "@app/core/common/enum/description-template-field-validation-type";
import { DescriptionTemplateStatus } from "@app/core/common/enum/description-template-status";
import { UserDescriptionTemplateRole } from "@app/core/common/enum/user-description-template-role";
import { BaseEntity } from "@common/base/base-entity.model";
import { Guid } from "@common/types/guid";
import { DescriptionTemplateType } from "../description-template-type/description-template-type";
import { User } from "../user/user";


export interface DescriptionTemplate extends BaseEntity {
	label: string;
	description: string;
	groupId: Guid;
	version: string;
	language: string;
	type: DescriptionTemplateType;
	status: DescriptionTemplateStatus;
	definition: DescriptionTemplateDefinition;
	users: UserDescriptionTemplate[];
}

export interface UserDescriptionTemplate extends BaseEntity {
	descriptionTemplate?: DescriptionTemplate;
	role?: UserDescriptionTemplateRole;
	user?: User;
}

export interface DescriptionTemplateDefinition {
	pages?: DescriptionTemplatePage[];
	sections?: DescriptionTemplateSection[];
}


export interface DescriptionTemplatePage {
	id: string;
	ordinal: number;
	title: string;
}

export interface DescriptionTemplateSection {
	id: string;
	ordinal: number;
	defaultVisibility: boolean;
	multiplicity: boolean;
	numbering: string;
	page: string;
	title: string;
	description: string;
	extendedDescription: string;

	sections?: DescriptionTemplateSection[];
	fieldSets: DescriptionTemplateFieldSet[];
}

export interface DescriptionTemplateFieldSet {
	id: string;
	ordinal: number;
	numbering: string;
	title: string;
	description: string;
	extendedDescription: string;
	additionalInformation: string;
	multiplicity: DescriptionTemplateMultiplicity
	hasCommentField: boolean;
	fields: DescriptionTemplateField[];
}

export interface DescriptionTemplateField {
	id: string;
	ordinal: number;
	numbering?: string;
	schematics?: string[];
	defaultValue?: string;
	visibilityRules?: DescriptionTemplateRule[];
	validations?: DescriptionTemplateFieldValidationType[];
	includeInExport: boolean;
	data?: DescriptionTemplateBaseFieldData;
}

export interface DescriptionTemplateRule {
	target: string;
	value: string;
}

export interface DescriptionTemplateMultiplicity {
	min: number;
	max: number;
	placeholder: string;
	tableView: boolean;
}

export interface DescriptionTemplateBaseFieldData {
	label: string;
	fieldType: DescriptionTemplateFieldType;
}


//
// Field Types
//
export interface DescriptionTemplateAutoCompleteData extends DescriptionTemplateBaseFieldData {
	multiAutoComplete: boolean;
	autoCompleteSingleDataList: DescriptionTemplateAutoCompleteSingleData[];
}

export interface DescriptionTemplateBooleanDecisionData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateCheckBoxData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateCurrencyData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateDataRepositoryData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateDatasetAutoCompleteData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateDatasetIdentifierData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateDatePickerData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateDmpAutoCompleteData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateExternalDatasetData extends DescriptionTemplateBaseFieldData {
	multiAutoComplete: boolean;
	type?: DescriptionTemplateFieldDataExternalDatasetType;
}

export interface DescriptionTemplateFreeTextData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplatePlaceholderAndMultiplicityData extends DescriptionTemplateBaseFieldData {
	multiAutoComplete: boolean;
}

export interface DescriptionTemplateLicenseData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateOrganizationData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplatePublicationData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateRadioBoxData extends DescriptionTemplateBaseFieldData {
	options: DescriptionTemplateRadioBoxOption[];
}

export interface DescriptionTemplateRegistryData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateResearcherAutoCompleteData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateResearcherData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateRichTextAreaData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateServiceData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateTagData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateTaxonomyData extends DescriptionTemplatePlaceholderAndMultiplicityData {
}

export interface DescriptionTemplateTextAreaData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateUploadData extends DescriptionTemplateBaseFieldData {
	types: DescriptionTemplateUploadOption[];
	maxFileSizeInMB: number;
}

export interface DescriptionTemplateValidationData extends DescriptionTemplateBaseFieldData {
}

export interface DescriptionTemplateWordListData extends DescriptionTemplateBaseFieldData {
	options: DescriptionTemplateComboBoxOption[];
	multiList: boolean;
}

//
// Others
//
export interface DescriptionTemplateAutoCompleteSingleData {
	autocompleteType: string;
	url: string;
	autoCompleteOptions: DescriptionTemplateComboBoxOption;
	optionsRoot: string;
	hasAuth: boolean;
	auth: DescriptionTemplateAuthAutoCompleteData
	method: string;
}

export interface DescriptionTemplateAuthAutoCompleteData {
	url: string;
	method: string;
	body: string;
	path: string;
	type: string;
}

export interface DescriptionTemplateComboBoxOption {
	label: string;
	value: string;
	source: string;
	uri: string;
}

export interface DescriptionTemplateRadioBoxOption {
	label: string;
	value: string;
}

export interface DescriptionTemplateUploadOption {
	label: string;
	value: string;
}