﻿import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormArray, UntypedFormGroup } from '@angular/forms';
import { BaseComponent } from '@common/base/base.component';
import { Guid } from '@common/types/guid';
import { DescriptionTemplateFieldEditorModel, DescriptionTemplateFieldSetEditorModel, DescriptionTemplateSectionEditorModel } from '../../description-template-editor.model';

@Component({
	selector: 'app-description-template-editor-section-component',
	templateUrl: './description-template-editor-section.component.html',
	styleUrls: ['./description-template-editor-section.component.scss']
})

export class DescriptionTemplateEditorSectionComponent extends BaseComponent implements OnInit {

	@Input() form: UntypedFormGroup;
	@Input() indexPath: string;
	@Input() viewOnly: boolean;
	@Output() fieldsetAdded = new EventEmitter<String>(); //returns the id of the fieldset added

	constructor() { super(); }

	ngOnInit() {

	}

	addField() {
		const fieldSet: DescriptionTemplateFieldSetEditorModel = new DescriptionTemplateFieldSetEditorModel();
		fieldSet.ordinal = (this.form.get('fieldSets') as UntypedFormArray).length;

		const field: DescriptionTemplateFieldEditorModel = new DescriptionTemplateFieldEditorModel();
		field.id = Guid.create().toString();
		field.ordinal = 0;//first field in fields
		fieldSet.fields.push(field);
		fieldSet.id = Guid.create().toString();

		const fieldsetsArray = this.form.get('fieldSets') as UntypedFormArray;
		fieldsetsArray.push(fieldSet.buildForm());

		const fieldSetForm = fieldsetsArray.at(fieldsetsArray.length - 1);
		//emit id inserted
		if (fieldSetForm) {
			const id: string = fieldSetForm.get('id').value;
			this.fieldsetAdded.emit(id);
		}
	}

	addSectioninSection() {
		const section: DescriptionTemplateSectionEditorModel = new DescriptionTemplateSectionEditorModel();
		//this.dataModel.sections.push(section);
		(<UntypedFormArray>this.form.get('sections')).push(section.buildForm());
	}

	DeleteSectionInSection(index) {
		//this.dataModel.sections.splice(index, 1);
		(<UntypedFormArray>this.form.get('sections')).removeAt(index);
	}

	deleteFieldSet(index) {
		//this.dataModel.fieldSets.splice(index, 1);
		(<UntypedFormArray>this.form.get('fieldSets')).removeAt(index);
	}

	keepPageSelectionValid(pagesJson: Array<any>) {
		const selectedPage = this.form.get('page').value as String;
		// const pages: Array<PageEditorModel> = JsonSerializer.fromJSONArray(pagesJson, Page);
		if (pagesJson.find(elem => elem.id === selectedPage) === undefined) {
			this.form.get('page').reset();
		}
	}

	getFieldTile(formGroup: UntypedFormGroup, index: number) {
		if (formGroup.get('title') && formGroup.get('title').value && formGroup.get('title').value.length > 0) { return formGroup.get('title').value; }
		return "Field " + (index + 1);
	}
}
