package eu.old.eudat.migration;

import eu.eudat.commons.enums.IsActive;
import eu.eudat.data.DescriptionReferenceEntity;
import eu.eudat.data.DmpReferenceEntity;
import eu.old.eudat.data.dao.entities.*;
import eu.old.eudat.data.entities.*;
import eu.old.eudat.logic.services.operations.DatabaseRepository;
import gr.cite.tools.data.query.QueryFactory;
import gr.cite.tools.logging.LoggerService;
import jakarta.persistence.EntityManager;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.List;
import java.util.UUID;

@Service
public class ReferenceMigrationService {

    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(DmpDatasetProfileMigrationService.class));
    private final DatabaseRepository databaseRepository;
    private final QueryFactory queryFactory;
    private static final int PageSize = 500;
    private static final boolean TestMode = false;
    private final EntityManager entityManager;

    public ReferenceMigrationService(DatabaseRepository databaseRepository, QueryFactory queryFactory, EntityManager entityManager) {
        this.databaseRepository = databaseRepository;
        this.queryFactory = queryFactory;
        this.entityManager = entityManager;
    }

    public void migrateDmpReferences() {
        migrateDmpOrganizations();
        migrateDmpResearchers();
        migrateDmpProjects();
        migrateDmpGrantsAndFounders();
    }

    public void migrateDatasetReferences() {
        migrateDatasetDataRepositories();
        migrateDatasetExternalDatasets();
        migrateDatasetRegistries();
        migrateDatasetServices();
    }

    public void migrateDmpOrganizations() {
        OrganisationDao organisationDao = databaseRepository.getOrganisationDao();
        long total = organisationDao.asQueryable().count();
        logger.debug("Migrate Dmp Organisation (from Organization) Total : " + total);
        int page = 0;

        List<Organisation> items;
        do {
            items = organisationDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dmp Organisation " + page * PageSize + " of " + total);

                for (Organisation item : items) {
//                    entityManager.detach(item);

                    for (DMP dmp : item.getDmps()) {
                        DmpReferenceEntity data = new DmpReferenceEntity();
                        data.setId(UUID.randomUUID());
                        data.setDmpId(dmp.getId());
                        data.setReferenceId(item.getId());
                        data.setCreatedAt(dmp.getCreated() != null ? dmp.getCreated().toInstant() : Instant.now());
                        data.setUpdatedAt(dmp.getModified() != null ? dmp.getModified().toInstant() : Instant.now());
                        data.setIsActive(IsActive.Active);
                        this.entityManager.persist(data);
                    }
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDmpResearchers() {
        ResearcherDao researcherDao = databaseRepository.getResearcherDao();
        long total = researcherDao.asQueryable().count();
        logger.debug("Migrate Dmp Researcher (from Researcher) Total : " + total);
        int page = 0;

        List<Researcher> items;
        do {
            items = researcherDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dmp Researcher " + page * PageSize + " of " + total);

                for (Researcher item : items) {
//                    entityManager.detach(item);

                    for (DMP dmp : item.getdMPs()) {
                        DmpReferenceEntity data = new DmpReferenceEntity();
                        data.setId(UUID.randomUUID());
                        data.setDmpId(dmp.getId());
                        data.setReferenceId(item.getId());
                        data.setCreatedAt(dmp.getCreated() != null ? dmp.getCreated().toInstant() : Instant.now());
                        data.setUpdatedAt(dmp.getModified() != null ? dmp.getModified().toInstant() : Instant.now());
                        data.setIsActive(IsActive.Active);
                        this.entityManager.persist(data);
                    }
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDmpProjects() {
        ProjectDao projectDao = databaseRepository.getProjectDao();
        long total = projectDao.asQueryable().count();
        logger.debug("Migrate Dmp Project (from Project) Total : " + total);
        int page = 0;

        List<Project> items;
        do {
            items = projectDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dmp Project " + page * PageSize + " of " + total);

                for (Project item : items) {
//                    entityManager.detach(item);

                    for (DMP dmp : item.getDmps()) {
                        DmpReferenceEntity data = new DmpReferenceEntity();
                        data.setId(UUID.randomUUID());
                        data.setDmpId(dmp.getId());
                        data.setReferenceId(item.getId());
                        data.setCreatedAt(dmp.getCreated() != null ? dmp.getCreated().toInstant() : Instant.now());
                        data.setUpdatedAt(dmp.getModified() != null ? dmp.getModified().toInstant() : Instant.now());
                        data.setIsActive(IsActive.Active);
                        this.entityManager.persist(data);
                    }
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDmpGrantsAndFounders() {
        GrantDao grantDao = databaseRepository.getGrantDao();
        long total = grantDao.asQueryable().count();
        logger.debug("Migrate Dmp Grants (from Grants) Total : " + total);
        int page = 0;

        List<Grant> items;
        do {
            items = grantDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("created"))).orderBy((builder, root) -> builder.asc(root.get("ID"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dmp Grants " + page * PageSize + " of " + total);

                for (Grant item : items) {
//                    entityManager.detach(item);

                    for (DMP dmp : item.getDmps()) {
                        DmpReferenceEntity data = new DmpReferenceEntity();
                        data.setId(UUID.randomUUID());
                        data.setDmpId(dmp.getId());
                        data.setReferenceId(item.getId());
                        data.setCreatedAt(dmp.getCreated() != null ? dmp.getCreated().toInstant() : Instant.now());
                        data.setUpdatedAt(dmp.getModified() != null ? dmp.getModified().toInstant() : Instant.now());
                        data.setIsActive(IsActive.Active);
                        this.entityManager.persist(data);
                        
                        if (item.getFunder() != null){
                            DmpReferenceEntity founder = new DmpReferenceEntity();
                            founder.setId(UUID.randomUUID());
                            founder.setDmpId(dmp.getId());
                            founder.setReferenceId(item.getFunder().getId());
                            founder.setCreatedAt(dmp.getCreated() != null ? dmp.getCreated().toInstant() : Instant.now());
                            founder.setUpdatedAt(dmp.getModified() != null ? dmp.getModified().toInstant() : Instant.now());
                            founder.setIsActive(IsActive.Active);
                            this.entityManager.persist(founder);
                        }
                    }
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDatasetDataRepositories() {
        DatasetDataRepositoryDao datasetDataRepositoryDao = databaseRepository.getDatasetDataRepositoryDao();
        long total = datasetDataRepositoryDao.asQueryable().count();
        logger.debug("Migrate Dataset DataRepository (from DatasetDataRepository) Total : " + total);
        int page = 0;

        List<DatasetDataRepository> items;
        do {
            items = datasetDataRepositoryDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("id"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dataset DataRepository " + page * PageSize + " of " + total);

                for (DatasetDataRepository item : items) {
//                    entityManager.detach(item);

                    DescriptionReferenceEntity data = new DescriptionReferenceEntity();
                    data.setId(UUID.randomUUID());
                    data.setDescriptionId(item.getDataset().getId());
                    data.setReferenceId(item.getDataRepository().getId());
                    data.setCreatedAt(item.getDataset().getCreated() != null ? item.getDataset().getCreated().toInstant() : Instant.now());
                    data.setUpdatedAt(item.getDataset().getModified() != null ? item.getDataset().getModified().toInstant() : Instant.now());
                    data.setIsActive(IsActive.Active);
                    this.entityManager.persist(data);
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDatasetExternalDatasets() {
        DatasetExternalDatasetDao datasetExternalDatasetDao = databaseRepository.getDatasetExternalDatasetDao();
        long total = datasetExternalDatasetDao.asQueryable().count();
        logger.debug("Migrate Dataset ExternalDataset (from DatasetExternalDataset) Total : " + total);
        int page = 0;

        List<DatasetExternalDataset> items;
        do {
            items = datasetExternalDatasetDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("id"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dataset ExternalDataset " + page * PageSize + " of " + total);

                for (DatasetExternalDataset item : items) {
//                    entityManager.detach(item);

                    DescriptionReferenceEntity data = new DescriptionReferenceEntity();
                    data.setId(UUID.randomUUID());
                    data.setDescriptionId(item.getDataset().getId());
                    data.setReferenceId(item.getExternalDataset().getId());
                    data.setCreatedAt(item.getDataset().getCreated() != null ? item.getDataset().getCreated().toInstant() : Instant.now());
                    data.setUpdatedAt(item.getDataset().getModified() != null ? item.getDataset().getModified().toInstant() : Instant.now());
                    data.setIsActive(IsActive.Active);
                    this.entityManager.persist(data);
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDatasetRegistries() {
        DatasetRegistryDao datasetRegistryDao = databaseRepository.getDatasetRegistryDao();
        long total = datasetRegistryDao.asQueryable().count();
        logger.debug("Migrate Dataset Registry (from DatasetRegistry) Total : " + total);
        int page = 0;

        List<DatasetRegistry> items;
        do {
            items = datasetRegistryDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("id"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dataset Registry " + page * PageSize + " of " + total);

                for (DatasetRegistry item : items) {
//                    entityManager.detach(item);

                    DescriptionReferenceEntity data = new DescriptionReferenceEntity();
                    data.setId(UUID.randomUUID());
                    data.setDescriptionId(item.getDataset().getId());
                    data.setReferenceId(item.getRegistry().getId());
                    data.setCreatedAt(item.getDataset().getCreated() != null ? item.getDataset().getCreated().toInstant() : Instant.now());
                    data.setUpdatedAt(item.getDataset().getModified() != null ? item.getDataset().getModified().toInstant() : Instant.now());
                    data.setIsActive(IsActive.Active);
                    this.entityManager.persist(data);
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

    public void migrateDatasetServices() {
        DatasetServiceDao datasetServiceDao = databaseRepository.getDatasetServiceDao();
        long total = datasetServiceDao.asQueryable().count();
        logger.debug("Migrate Dataset Service (from DatasetService) Total : " + total);
        int page = 0;

        List<DatasetService> items;
        do {
            items = datasetServiceDao.asQueryable().orderBy((builder, root) -> builder.asc(root.get("id"))).skip(page * PageSize).take(PageSize).toList();
            if (items != null && !items.isEmpty()) {
                logger.debug("Migrate Dataset Service " + page * PageSize + " of " + total);

                for (DatasetService item : items) {
//                    entityManager.detach(item);

                    DescriptionReferenceEntity data = new DescriptionReferenceEntity();
                    data.setId(UUID.randomUUID());
                    data.setDescriptionId(item.getDataset().getId());
                    data.setReferenceId(item.getService().getId());
                    data.setCreatedAt(item.getDataset().getCreated() != null ? item.getDataset().getCreated().toInstant() : Instant.now());
                    data.setUpdatedAt(item.getDataset().getModified() != null ? item.getDataset().getModified().toInstant() : Instant.now());
                    data.setIsActive(IsActive.Active);
                    this.entityManager.persist(data);
                }
                this.entityManager.flush();

                page++;
            }
        } while (items != null && !items.isEmpty());
    }

}
